﻿'use strict';
(function (_win) {
    var $t = (function () {
        //eat: Acronym for Enterprise Admin Tool/Tag Administrator
        var eat = function () {
        };

        eat.version = "2.0.2";

        // Config the item that shows in the same line with other item, it supports multiple item and separated with comma
        // There are three parameters
        // First: WTConfigKey of item which is the base item
        // Second: WTConfigKey of item which will show in the same line of UI behide the base item
        // Third: true: show element control firstly, and then a label; false: show the label firstly, and then the element control
        eat.wtConfigOnOneLine = "homedom:homedomRegExp:true";

        //---------------------------Common----------------------------------//
        eat.isNull = function (_val) {
            return typeof _val == 'undefined' || _val == null;
        };

        eat.isEmpty = function (_val) {
            return /^\s*$/.test(_val);
        };

        //---------------------------Msg Mapper-------------------------//
        eat.mapper = {
            msgs: {
                "The remote server returned an error: (401) Unauthorized.": "The sign-in name or password does not match one in the system. Error Code: 401 Unauthorized",
                "The remote server returned an error: (403) Forbidden.": "Unable to access this domain. Error Code: 403 Forbidden"
            },
            getMsg: function (_key) {
                if (typeof this.msgs[_key] != "undefined") {
                    return this.msgs[_key];
                } else {
                    return _key;
                }
            }
        };

        //---------------------------Log--------------------------------//
        eat.logger = {
            bc: false, //show msg on browser console when using $t.log function
            createItem: function (type, subject, detail) {
                //type:error,info,warning
                return { Type: type, Subject: subject, Detail: detail, DateTime: new Date() }
            },
            appendItem: function (_item) {                
                var row = $("<div class=\"status_item\"><img class=\"item_image\" src=\"/images/info.png\" /><div class=\"item_title\"></div><div class=\"item_date\"></div><div class=\"item_details\"></div></div>");
                row.find(".item_image").attr("src", "/images/" + _item.Type + ".png");
                row.find(".item_title").text(_item.Subject);
                row.find(".item_date").text(_item.DateTime.toLocaleDateString() + " " + _item.DateTime.toLocaleTimeString());
                row.find(".item_details").text(_item.Detail);                
                $("#status_panel_content").append(row);
                $("#btn_status").removeClass("icon_info").addClass("icon_alert");
            },
            appendResultMsg: function (resultMsg) {
                this.append(resultMsg.MsgType, resultMsg.MsgSubject, resultMsg.MsgContent)
            },
            append: function (type, subject, detail) {
                this.appendItem(this.createItem(type, subject, detail));
            },
            err: function (subject, detail) {
                this.append("error", subject, detail);
            },
            info: function (subject, detail) {
                this.append("info", subject, detail);
                $("#btn_status").removeClass("icon_alert").addClass("icon_info");
            },
            warn: function (subject, detail) {
                this.append("warning", subject, detail);
            },
            open: function () {
                $(".status_view_popup").hide();
                $("#status_view_popup").show();
                $("#btn_status").removeClass("icon_alert");
            },
            close: function () {
                $("#status_view_popup").hide();
            },
            clear: function () {
                $("#status_panel_content").empty();
            },
            clearCache: function () {
                $t.log("Clear cache ...");
                $t.mask.open("In Progress ...")
                $.ajax({
                    type: "POST",
                    url: "/Default/ClearCache",
                    success: function (data, _textStatus) {
                        //Do nothing 
                    },
                    error: function (_XMLHttpRequest, _textStatus, _errorThrown) {
                        //Do nothing 
                    },
                    complete: function (_XMLHttpRequest, _textStatus) {
                        $t.mask.close();
                        $t.log("Clear cache completed. " + _textStatus);
                    }
                })
            },
            init: function () {
                this.clear();
                $("#btn_status").click(function () {
                    $t.logger.open();
                });
                $("#btn_cache_clear").click(function () {
                    $t.logger.clearCache();
                });
                $("#btn_status_clear").click(function () {
                    $t.logger.clear();
                });
                $("#btn_status_close").click(function () {
                    $t.logger.close();
                });
            }
        };

        eat.log = function (_msg, _showOnPage) {
            if ($t.logger.bc) {
                console.log("["+(new Date()).toLocaleTimeString()+"] "+_msg);
            }
            if (_showOnPage) {
                $t.logger.info(_msg, _msg);
            }
        };

        //---------------------------UI-------------------------------------//
        eat.hasSelection = function () {
            return $(".ckb_sc:checked:not(:hidden)").length != 0;
        };

        eat.alert = function (_text) {
            var _msg = $("#__msg_");
            if (_msg.length == 0) {
                _msg = $("<div id='__msg_'></div>").appendTo("<body/>");
                _msg.dialog({
                    title: "Webtrends",
                    autoOpen: false,
                    resizable: true,
                    width: 360,
                    height: 200,
                    modal: true,
                    buttons: {
                        "OK": {
                            click: function () {
                                $(this).dialog("close");
                            },
                            text: 'OK',
                            class: 'btn_orange'
                        }
                    }
                });
            }

            _msg.empty().html("<span class=\"ui-icon ui-icon-alert\" style=\"float:left; margin:2px 7px 20px 20px;\"></span>" + _text);
            _msg.dialog("open");

        };

        eat.prompt = {
            err: function (message, args) {
                $("#err").html($("#err").html() + "<li>" + message + "</li>");
                if (args != null) $("#debug").html($("#debug").html() + "<li>" + args.get_message() + "</li>");
            },

            clean: function () {
                $("#err").html("");
            }
        };

        eat.pg = {
            //default max value is 8
            create: function (_id) {
                this.remove(_id);
                var _pg = $("<div id=\"pg_" + _id + "\" class=\"pg_body\"><div id=\"pg_" + _id + "_label\" class=\"pg_label\"></div></div>");
                $(_pg).progressbar({ max:8 })
                    .appendTo('#WTSite_' + _id + "_State");
            },

            remove: function (_id) {
                $('#WTSite_' + _id + "_State").empty();                
            },

            update: function (_id,_option) {
                $("#pg_" + _id).progressbar("option", _option);
            },

            progress: function (_id, _val) {
                $("#pg_" + _id).progressbar("option", "value", _val);
                $("#pg_" + _id + "_label").text(((parseInt(_val) / parseInt($("#pg_" + _id).progressbar("option", "max"))) * 100) + "%");
            },

            clean: function () {
                $(".sc_state").empty();
            },

            createByUrl: function (_url) {
                var site = $t.globals.siteCollections[_url];
                if (site) {
                    this.create(site.Id);
                }
            },

            updateByUrl: function (_url,_option) {
                var site = $t.globals.siteCollections[_url];
                if (site) {
                    this.update(site.Id, _option);
                }
            },

            removeByUrl: function (_url) {
                var site = $t.globals.siteCollections[_url];
                if (site) {
                    this.remove(site.Id);
                }
            },

            progressByUrl: function (_url, _val) {
                var site = $t.globals.siteCollections[_url];
                if (site) {
                    this.progress(site.Id, _val);
                    if ($("#pg_" + site.Id + "_label").text() == "100%") {
                        this.success(_url, "Process has been completed.");
                    }
                }
            },

            warn: function (_url, _msg) {                
                var site = $t.globals.siteCollections[_url];
                if (site) {
                    this.remove(site.Id);
                    $('#WTSite_' + site.Id + "_State").append($("<img/>",{
                        class: "pg_err",
                        alt: _msg,
                        id: "pg_" + site.Id + "_err",
                        src: "/images/warning.png"
                    })).attr("title", _msg);
                    
                }
            },

            success: function (_url, _msg) {
                var site = $t.globals.siteCollections[_url];
                if (site) {
                    this.remove(site.Id);
                    $('#WTSite_' + site.Id + "_State").append($("<img/>", {
                        class: "pg_err",
                        alt: _msg,
                        id: "pg_" + site.Id + "_err",
                        src: "/images/success.png"
                    })).attr("title", _msg);
                }
            },

            failed: function (_url, _msg) {                
                var site = $t.globals.siteCollections[_url];
                if (site) {
                    this.remove(site.Id);
                    $('#WTSite_' + site.Id + "_State").append($("<img/>",{
                            class: "pg_err",
                            alt: _msg,
                            id: "pg_" + site.Id + "_err",
                            src: "/images/error.png"
                    })).attr("title", _msg);
                    
                }
            }
        };

        eat.mask = {
            _init: false,            
            _dialog: null,
            _progress: null,
            init: function () {
                if (!this._init) {
                    this._init = true;
                    var _win = $("<div class='mask'/>").appendTo("body");
                    this._dialog = _win.dialog({
                        title: "In progress ...",
                        draggable: false,
                        autoOpen: true,
                        resizable: false,
                        //height: 60,
                        width: 600,
                        modal: true,
                        closeOnEscape: false
                        //close: function (event, ui) { $(event.target).empty(); }
                    });
                   
                    var _pgb = $("<div class='progressbar'><div class='progress_label'></div></div>").appendTo(_win);
                    
                    this._progress = _pgb.progressbar({
                        value: false
                    });                    

                    this._dialog.siblings(".ui-dialog-titlebar").hide()
                }                
                
            },
            open: function (_msg) {
                this.init();
                this._dialog.dialog("open");
                this._dialog.dialog("option", {height:23});
                this._dialog.find(".ui-progressbar-value").css({
                    "background": 'yellow'
                    //"background": '#' + Math.floor(Math.random() * 16777215).toString(16)
                });
                this._progress.find(".progress_label").empty();
                if (_msg) {
                    this._progress.find(".progress_label").text(_msg);
                }
            },
            close: function () {
                this.init();
                this._dialog.dialog("close");
            }
        };
       
        eat.storage = {
            getInfo:function(){
                var info = localStorage.getItem("Webtrends_eat_info");
                if (!info) {
                    info = "{}";
                }
                return JSON.parse(info);
            },

            saveInfo: function (info) {
                if (!info) {
                    info = {};
                }
                localStorage.setItem("Webtrends_eat_info", JSON.stringify(info));
            },

            putItem: function (_key,_val) {
                var info = $t.storage.getInfo();
               
                info[_key] = _val;               

                $t.storage.saveInfo(info);
            },

            loadLogin: function () {
                var info = $t.storage.getInfo();
                if (info) {
                    if ($(":radio[name=selectRoute]").length > 1) {
                        $("#siteFullUrl").val(info["adminURL"]);
                        $("#usr").val(info["account"]);
                        $("#route" + info["route"]).prop("checked", true);
                    } else {
                        var _route = $(":radio[name=selectRoute]").val();
                        if (_route) {
                            $t.storage.changeRoute(_route);
                        }                        
                    }
                    
                }
            },

            saveLogin: function () {                              
                $t.storage.putItem("adminURL", $("#siteFullUrl").val().trim());
                $t.storage.putItem("account", $("#usr").val().trim());
                $t.storage.putItem("route", $(":radio[name=selectRoute]:checked").val().trim());
                $t.storage.putItem($(":radio[name=selectRoute]:checked").val(), {
                    "adminURL": $("#siteFullUrl").val(),
                    "account": $("#usr").val()
                })                
            },

            changeRoute: function (_route) {
                var info = $t.storage.getInfo();
                if (info) {
                    var routeDetail = info[_route];
                    if (routeDetail) {
                        $("#siteFullUrl").val(routeDetail["adminURL"]);
                        $("#usr").val(routeDetail["account"]);
                    }
                }
            }

            
        };

        //======================================EAT initialization=====================================//
        eat._init = false;
        eat.init = function () {
            $t.log("Init the context & settings.");

            //Globals
            $t.globals.login = "N";
            $t.globals.siteCollections = [];
            $t.globals.siteInUse = "";
            $t.globals.batch = { batchNo: "", sampleURL: "", sampleIdx: 0, sampleState: "", sites: [] };
            $t.globals.filter = null;
            $t.globals.furtive = false;
            

            //Below Actions should be init once only
            if (!$t._init) {
                $t._init = true;

                $(_win).resize(function () {
                    //todo
                })


                //Bind keypress on password input box to trigger login
                $('#pwd').bind('keypress', function (event) {
                    if (event.keyCode == "13") {
                        $t.login();
                    }
                });

                //Login Event
                $("#btn_login").click($t.login);

                //Logout Event
                $("#btn_logout").click($t.logout);                

                //Search 
                $("#btn_search").click($t.search);

                //Bind keypress for search
                $('#searchText').bind('keypress', function (event) {
                    if (event.keyCode == "13") {
                        $t.search();
                    }
                });

                //Batch Refresh
                $("#btn_batch_refresh").click($t.refreshSites);                

                $("#moreCondition").click($t.conditionFilter);

                //Initialize all sub-modules
                for (var p in $t) {
                    if (typeof ($t[p]) == "object" && typeof ($t[p]["init"]) == "function") {
                        $t[p]["init"]();
                    }
                }

                //Configuration Dialog
                $("#waaConfig").dialog({
                    autoOpen: false,
                    resizable: true,
                    width: 840,
                    height: 640,
                    modal: true
                });

                //save config
                $("#saveconfig").click($t.saveConfigs);

                //retract WAA
                $("#uninstallwt").click($t.uninstallWebtrends);

                //checkbox
                $("#ckb_sc_all").click(function () {
                    if ($(this).is(":checked")) {
                        $(".ckb_sc").each(function () {
                            if ($(this).is(":hidden") || $(this).is(":disabled")) {
                                $(this).prop("checked", false);
                                $(this).parents("tr").removeClass("selected");
                            } else {
                                $(this).prop("checked", true);
                                $(this).parents("tr").addClass("selected");
                            }
                        });
                    } else {
                        $(".ckb_sc").prop("checked", false);
                        $(".ckb_sc").parents("tr").removeClass("selected");
                    }
                    $t.batch.check();
                });

                $("#ckb_waa_installed").click(function () {
                    //Before loading sites, make 'more' button disabled
                    $("#link_sc_more").addClass("btn_gray").attr('disabled', true);

                    if (!$(this).is(":checked")) {
                        $("#ckb_tracking_enabled").prop("checked", false);
                        $("#ckb_tracking_disabled").prop("checked", false);
                        $("#ckb_ups_enabled").prop("checked", false);
                        $("#ckb_ups_disabled").prop("checked", false);                        
                        $("#ckb_valid_dcs").prop("checked", false);
                        $("#ckb_invalid_dcs").prop("checked", false);                        
                    }
                    if ($t.globals.filter && $t.globals.filter.id) {
                        clearTimeout($t.globals.filter.id);
                    }
                    $t.globals.filter = {};
                    $t.globals.filter.id = setTimeout($t.search, 2000);
                });
                
                $("#ckb_waa_not_installed").click(function () {
                    //Before loading sites, make 'more' button disabled
                    $("#link_sc_more").addClass("btn_gray").attr('disabled', true);

                    if ($t.globals.filter && $t.globals.filter.id) {
                        clearTimeout($t.globals.filter.id);
                    }
                    $t.globals.filter = {};
                    $t.globals.filter.id = setTimeout($t.search, 2000);
                });

                $("#ckb_tracking_enabled, #ckb_tracking_disabled, #ckb_valid_dcs, #ckb_invalid_dcs, #ckb_ups_enabled, #ckb_ups_disabled").click($t.filter);

                //More button
                $("#link_sc_more").click($t.loadSites);

                //Init config file upload
                $("#configImportForm").ajaxForm({
                    beforeSubmit: function () {
                        if ($("#configFile").get(0).files.length === 0) {
                            $t.alert("Please select the configuration file.");
                            return false;
                        }
                        return true;                            
                    },
                    beforeSend: function () {                        
                        $t.mask.open("System is uploading the file, please stand by.");                        
                    },
                    uploadProgress: function (event, position, total, percentComplete) {
                        //Do nothing
                    },
                    success: function (data) {                        
                        data = $t.parseData(data);
                        if ($t.checkData(data)) {
                            //If data is correct, close the import dialog
                            $("#configImportDialog").dialog("close");
                            
                            //Parse the config object from XML
                            var waaConfig = data;
                            for(var i in waaConfig.Groups){
                                var group = waaConfig.Groups[i];
                                var groupTitle = group.Title;
                                for (var j in group.Items) {
                                    var item = group.Items[j];
                                    
                                    //If the group is for User Profile
                                    if (groupTitle == 'User Profile') {
                                        if ("extraUserInfo" == item.ID) {
                                            $('#WTConfig' + item.ID).prop("checked", item.Value == "true");
                                        } else {
                                            $('#WTUPField' + item.ID + '_enabled').prop("checked", item.Collect);
                                            $('#WTUPField' + item.ID + '_param').val(item.Value != null ? item.Value : '');
                                        }                                        
                                    } else if (groupTitle == "Plugins") {
                                        var pluginEnabled = $('#WTPlugin' + item.ID + '_enabled');
                                        if (pluginEnabled && pluginEnabled.is(":checkbox")) {
                                            pluginEnabled.prop("checked", item.Enabled == true);
                                        }

                                        var pluginOptions = $('#WTPlugin' + item.ID + '_options');
                                        if (pluginOptions) {
                                            pluginOptions.val(item.Value);
                                        }
                                    } else {                                        
                                        var wtconfigObj = $('#WTConfig' + item.ID);

                                        if (wtconfigObj.is(":checkbox")) {
                                            wtconfigObj.prop("checked", item.Value == "true");
                                        }else if (wtconfigObj.is(":text")) {
                                            wtconfigObj.val(item.Value);
                                        }else if (wtconfigObj.is("select")) {
                                            wtconfigObj.find("option[value='" + item.Value + "']").attr("selected", true);
                                        } else if (wtconfigObj.is("textarea")) {
                                            wtconfigObj.text(item.Value);
                                        }
                                    }
                                }
                            }

                            $t.saveConfigs();

                        } else {
                            //Close the progress bar if there's any issue
                            $t.mask.close();
                            //Show error message
                            $t.alert("Upload configuration file failed. " + (data && data.MsgContent) ? data.MsgContent : "");
                        }
                    },
                    complete: function (xhr) {                        
                        $t.log("Import config completed, " + xhr.responseText);
                        $t.mask.close();
                    }
                });

                $("#configImportDialog").dialog({
                    autoOpen: false,
                    resizable: true,
                    width: 340,
                    height: 80,
                    modal: true
                });

                $("#importConfig").click(function () {
                    $("#configImportDialog").dialog("open");
                });

                $(":radio[name=selectRoute]").change(function () {
                    $t.storage.changeRoute($(this).val());
                });
            }
            //UI init
            $(".home").hide();
            $(".login").show();
            $("body").addClass("login_bg").removeAttr("style");
            $("#sc_table").find("tbody").empty();
            $("#waaConfig").dialog("close");
            $t.mask.close();
            $("#sc_page_info").empty();
            $("#ckb_waa_installed").prop("checked", true);
            $("#ckb_waa_not_installed").prop("checked", true);
            $("#ckb_sc_all").prop("checked", false);
            $(".status_view_popup").hide();
            $("#searchText").val("");

            //attributes should be undefined when the page is initialized
            $("#link_sc_more").removeAttr("Index");
            $("#link_sc_more").removeAttr("Authed");
            $("#link_sc_more").removeAttr("Search");

            $t.storage.loadLogin();

            //Load version number from assembly
            $t.getVersion();
        };

        //Open config dialog
        eat.openConfig = function () {
            $t.prompt.clean();
            $("#configTabs").hide();
            $("#tabs").siblings().remove();
            if ($t.globals.furtive == false) {
                $("#waaConfig").dialog("open");
            }

            //Hide export button
            $("#exportConfig").hide();

            //If the batch update, it should be empty
            $t.globals.siteInUse = "";
        };

        //It should be called before save configuration
        eat.cleanData = function () {
            //clean messages
            $t.prompt.clean();

            //Reomve Progress bars
            $t.pg.clean();

            //Globals Rest
            $t.globals.batch = { batchNo: "", sampleURL: "", sampleIdx: 0, sampleState: "", sites: [] };
        };

        //---------------------------Business--------------------------------//
        //Globals
        eat.globals = {
            login: "N", //N:not login/login failed, I: is loging into, S: login success, 
            siteCollections: [], //site collections cache  
            batch: { batchNo: "", sampleURL: "", sampleIdx: 0, sampleState: "", sites: [] },
            siteInUse: "", //the site url in use
            filter: null,
            furtive: false
        };

        //Convert Data from backend, normally, the data should has JSON format.
        eat.parseData = function (_data) {
            if (typeof _data != "undefined") {
                if (typeof _data == "string") {
                    if (_data.length > 0) {
                        return JSON.parse(_data);
                    }
                }
                if (typeof _data == "object") {
                    return _data;
                }
            }
            return null;
        };

        //Check Data from backend is Message information or not.
        //e.g. SiteEntity, ResultMsg
        eat.checkData = function (_data) {
            _data = $t.parseData(_data);
            if (_data != null) {
                if (_data.MsgType) {
                    $t.logger.append(_data.MsgType, _data.MsgSubject, _data.MsgContent);
                    if (_data.MsgType == "info" && _data.MsgCatalog == 'success') {
                        return true;
                    }
                    if (_data.MsgType == "error" && _data.MsgCatalog == 'timeout') {
                        //Logout
                        //$t.init();
                        history.go(0);
                    }
                    return false;
                }
                return true;
            }
            return false;
        };

        //Handle Error
        eat.handleError = function (_url, _msg) {
            var __msg = _msg;
            if (typeof _msg == 'object' && _msg instanceof Array) {
                __msg = "";
                for (var i = 0; i < _msg.length; i++) {
                    __msg += "["+ _msg[i].MsgSubject + " | " +_msg[i].MsgContent+"]";
                }
            }
            //Remove the progress bar and add the error icon
            $t.pg.failed(_url, __msg);
            //if it is the sample of the batch, set the state value
            if ($t.globals.batch.batchNo != "") {
                if ($t.globals.batch.sampleURL == _url) {
                    $t.globals.batch.sampleState = "error";                    
                }
            }
        };

        //Handle Warning
        eat.handleWarning = function (_url, _msg) {
            var __msg = _msg;
            if (typeof _msg == 'object' && _msg instanceof Array) {
                __msg = "";
                for (var i = 0; i < _msg.length; i++) {
                    __msg += "[" + _msg[i].MsgSubject + " | " + _msg[i].MsgContent + "]";
                }
            }
            //Remove the progress bar and add the error icon
            $t.pg.warn(_url, __msg);
            //if it is the sample of the batch, set the state value
            if ($t.globals.batch.batchNo != "") {
                if ($t.globals.batch.sampleURL == _url) {
                    $t.globals.batch.sampleState = "warn";
                }
            }
        };

        //Logout
        eat.logout = function () {
            if (_win.confirm("Are you sure to log out?")) {
                $t.log("Log out ...");
                $t.mask.open("In Progress ...")
                $.ajax({
                    type: "POST",
                    url: "/Default/Logout",
                    success: function (data, _textStatus) {
                        //Do nothing 
                    },
                    error: function (_XMLHttpRequest, _textStatus, _errorThrown) {
                        //Do nothing 
                    },
                    complete: function (_XMLHttpRequest, _textStatus) {
                        $t.log("Log out completed. " + _textStatus);
                        //window.location.reload();  //Firefox has issue
                        //window.location.href = window.location.href;
                        //history.go(0);
                        window.location.reload(true); // Force to refresh
                        //$t.init();
                    }
                })
            }
            
        };

        //Load version from backend
        eat.getVersion = function () {
            $t.log("Load version from backend ...");
            $.ajax({
                type: "GET",
                url: "/Default/getVersion",
                success: function (data, _textStatus) {
                    data = $t.parseData(data);
                    $t.log("System version is " + data.version);
                    $t.version = data.version;
                    $("#status_view_popup .version_val").empty().text(data.version);
                },
                error: function (_XMLHttpRequest, _textStatus, _errorThrown) {
                    //Do nothing      
                },
                complete: function (_XMLHttpRequest, _textStatus) {
                    //Do nothing                      
                }

            })
        };

        //Login        
        eat.login = function () {            
            if ($t.globals.login != "N") return;
            $t.globals.login = "I";

            //validation
            var _url = $("#siteFullUrl").val().trim(), _usr = $("#usr").val().trim(), _pwd = $("#pwd").val().trim(), _route = $(":radio[name=selectRoute]:checked").val();
            if ($t.isEmpty(_url)) {
                $t.globals.login = "N";
                $t.alert("'Admin URL' is required.");
                return;
            }
            else
            {
                var regex = "(http|https):\/\/([\w.]+\/?)\*";
                if(!_url.match(regex)){
                    $t.globals.login = "N";
                    $t.alert("The protocol in the URL is missing.");
                    return;
                }
            }

            if ($t.isEmpty(_usr)) {
                $t.globals.login = "N";
                $t.alert("'Account' is required.");
                return;
            }

            if ($t.isEmpty(_pwd)) {
                $t.globals.login = "N";
                $t.alert("'Password' is required.");
                return;
            }

            $("#loading").show();

            $t.log("Log into system ...");
            $.ajax({
                type:"POST",
                url: "/Default/Login",
                data: { url: _url, username: _usr, password: _pwd, route: _route },
                success: function (data, _textStatus) {
                    $("#loading").hide();
                    data = $t.parseData(data);
                    if ($t.checkData(data)) {                        
                        //UI change
                        $(".login").hide();
                        $(".home").show();
                        $("body").css("background", "#fff").removeClass("login_bg");

                        //adjust the list table size and attach scroll event to it
                        var wH = $(window).height();
                        var warpH = wH - 230;
                        $(".sc_content_wrap").height(warpH);
                        $(".sc_content_wrap").on('scroll', function () {
                            if ($(this).scrollTop() + $(this).innerHeight() >= $(this)[0].scrollHeight) {
                                if (!$("#link_sc_more").attr('disabled')) {
                                    $t.loadSites();
                                }                                
                            }
                        });

                        $t.loadSites();
                        
                        $t.globals.login = "S";

                        $t.storage.saveLogin();
                      
                    } else {
                        var _msg = (data && data.MsgContent) ? data.MsgContent : "Login failed."
                        $t.alert($t.mapper.getMsg(_msg));
                        $t.globals.login = "N";
                    }
                },
                error: function (_XMLHttpRequest, _textStatus, _errorThrown) {
                    $("#loading").hide();
                    $t.alert("Login failed.");
                    $t.globals.login = "N";
                },
                complete: function (_XMLHttpRequest, _textStatus) {
                    //Do nothing  
                    $t.log("Log into system completed. " + _textStatus);
                }

            })
        };

        //Load site to refresh the app status
        eat.loadSite = function (_url, _appLoad) {
            $.ajax({
                type: "POST",
                cache: false,
                context: { site: _url, appLoad: (_appLoad ? _appLoad : true) },
                url: "/Default/LoadSite",
                data: { url: _url },
                success: function (data, _textStatus) {                    
                    data = $t.parseData(data);
                    if ($t.checkData(data)) {
                        $t.showSite(data);

                        if (this.appLoad && data.IsWAAInstalled == 1) {
                            $t.loadApp(data.Url);
                        }
                    } 
                },
                error: function (_XMLHttpRequest, _textStatus, _errorThrown) {
                    //todo:mask
                    $t.alert("Load site failed.");
                },
                complete: function (_XMLHttpRequest, _textStatus) {
                    $t.log("Load site completed. " + _textStatus);
                }

            })
        };

        eat.cleanBtns = function (_siteId) {
            $('#WTSite_' + _siteId + "_App").empty();
            $('#WTSite_' + _siteId + "_Tracking").empty();
            $('#WTSite_' + _siteId + "_Ups").empty();
        };

        //Build the refresh button in the "WAA App" column
        eat.buildRefreshBtn = function (_url) {
            var _site = $t.globals.siteCollections[_url];
            //$('#WTSite_' + _site.Id + "_App").empty();
            $t.cleanBtns(_site.Id);
            $("<img/>", {
                class: "pg_err",
                alt: "Refresh",
                src: "/images/refresh.png",
                site: _site.Url,
                siteId: _site.Id
            }).click(function () {
                var __url = $(this).attr("site");
                var __id = $(this).attr("siteId");
                $t.buildWaitingCol(__id);

                $t.loadSite(__url);
            }).appendTo('#WTSite_' + _site.Id + "_App");

        };

        //Refresh Sites to load the installation status
        eat.refreshSites = function () {
            //Uncheck 'Select All'
            $("#ckb_sc_all").prop("checked", false);

            $(".ckb_sc:checked:not(:hidden)").each(function () {
                //todo:check in use                
                $t.refreshSite($(this).attr("_url"));
            });
            $t.batch.check();
        };

        eat.getConditions = function (forTitle) {
            var conditionArr = [];
            if ($("#ckb_tracking_enabled").is(":checked") && !$("#ckb_tracking_disabled").is(":checked")) {
                conditionArr.push(forTitle?"Tracking Enabled":"TrackEnabled");                
            }

            if (!$("#ckb_tracking_enabled").is(":checked") && $("#ckb_tracking_disabled").is(":checked")) {
                conditionArr.push(forTitle ? "Tracking Disabled" : "TrackDisabled");
            }

            if ($("#ckb_ups_enabled").is(":checked") && !$("#ckb_ups_disabled").is(":checked")) {
                conditionArr.push(forTitle ? "User Profile Service Enabled" : "UserProfileCollectionEnabled");
            }

            if (!$("#ckb_ups_enabled").is(":checked") && $("#ckb_ups_disabled").is(":checked")) {
                conditionArr.push(forTitle ? "User Profile Service Disabled" : "UserProfileCollectionDisabled");
            }

            if ($("#ckb_valid_dcs").is(":checked") && !$("#ckb_invalid_dcs").is(":checked")) {
                conditionArr.push(forTitle ? "Valid DCSID" : "ValidDCSEnabled");
            }

            if (!$("#ckb_valid_dcs").is(":checked") && $("#ckb_invalid_dcs").is(":checked")) {
                conditionArr.push(forTitle ? "Invalid DCSID" : "InvalidDCSEnabled");
            }

            return conditionArr;
        };

        eat.updateSearchTitle = function () {
            var conditionArr1 = [];
            if ($("#ckb_waa_installed").is(":checked")) {
                conditionArr1.push("App Deployed");
            }

            if ($("#ckb_waa_not_installed").is(":checked")) {
                conditionArr1.push("App Not Deployed");
            }

            var conditionArr2 = $t.getConditions(true);

            $("#search_title_sum").html(conditionArr1.concat(conditionArr2).join());
        };

        //To call the service filter function to filter the site collections
        eat.filterExecution = function () {            
            $t.globals.filter = null;

            var conditionArr = $t.getConditions();

            //Append ALL site urls on page which has WAA installed for filter
            var urlArr = [];
            $(".ckb_sc[waaIsInstalled=1]").each(function () {
                urlArr.push($(this).attr("_url"));
            });

            if (conditionArr.length == 0) {                
                //If no condition, just check the abvove two checkboxes
                if ($("#ckb_waa_installed").is(":checked")) {                    
                    $t.displaySiteRow(true, true);
                } else {
                    $t.displaySiteRow(true, false);
                }

                if ($("#ckb_waa_not_installed").is(":checked")) {                    
                    $t.displaySiteRow(false, true);
                } else {
                    $t.displaySiteRow(false, false);
                }

                $t.batch.check();
                
                return;
            }

            if (urlArr.length == 0) {
                //If no sites, do not call service to filter

                return;
            }

            $t.mask.open("System is filtering site collections, please stand by.");
            $.ajax({
                type: "POST",
                url: "/Default/FilterSites",
                contentType: "application/json",
                data: JSON.stringify({ "conditions": conditionArr, "urls": urlArr }),
                success: function (data) {
                    data = $t.parseData(data);

                    if ($t.checkData(data)) {
                        $("tr.site_row").hide();
                        //Data is the array object which contains the site id
                        for (var i in data) {
                            var _siteId = data[i];                             
                            $("#WTSite_" + _siteId + "_row").show();
                        }

                        var totalstr = $("tr.site_row:not(:hidden)").length;
                        if (totalstr != "0") {
                            totalstr = "1-" + totalstr + " of " + _pagination.Total;
                        }
                        //TODO
                        $("#sc_page_info").text("Total:" + totalstr);
                    }
                   
                },
                complete: function (_XMLHttpRequest, _textStatus) {
                    $t.mask.close();
                    $t.batch.check();
                }
            });
        };

        //Show or Hide the site row & control the status of the checkbox
        eat.displaySiteRow = function (waaIsInstalled, showIt) {
            var _installed = waaIsInstalled ? "1" : "0";
            if (showIt) {
                $("tr[waaIsInstalled=" + _installed + "]").show();
            } else {
                $("tr[waaIsInstalled=" + _installed + "]").hide();
                //$("tr[waaIsInstalled=" + _installed + "]:hidden :checkbox:checked").prop("checked", false);
            }
            
        };

        //Call the query delay for 2 seconds
        eat.filter = function () {
            //Before loading sites, make 'more' button disabled
            $("#link_sc_more").addClass("btn_gray").attr('disabled', true);

            if ($(this).is(":checked")) {
                //$("#ckb_waa_not_installed").prop("checked", false);
                $("#ckb_waa_installed").prop("checked", true);

                //Hide all site collections which has no waa installed
                //$t.displaySiteRow(false, false);
            }

            if ($t.globals.filter && $t.globals.filter.id) {
                clearTimeout($t.globals.filter.id);
            }
            $t.globals.filter = {};
            $t.globals.filter.id = setTimeout($t.search, 2000);
        };
        
        //UI change when more condition is shown.
        eat.conditionFilter = function () {
            var hCD = $("#hideCondition");
            var mCD = $("#moreCondition");
            hCD.is(":hidden") ? (hCD.show(100), mCD.addClass("arrow_up")) : (hCD.hide(100), mCD.removeClass("arrow_up"));
        };

        //Force reload the site status
        eat.refreshSite = function (_url) {
            var _site = $t.globals.siteCollections[_url];
            $t.buildWaitingCol(_site.Id);
            //Reset
            $('#WTSite_' + _site.Id + "_enabled").prop("checked", false);                                                
            $('#WTSite_' + _site.Id + "_enabled").attr("disabled", true);

            $.ajax({
                type: "POST",
                cache: false,
                context: _url,
                url: "/Default/RefreshSite",
                data: { url: _url },
                success: function (data, _textStatus) {
                    data = $t.parseData(data);
                    if ($t.checkData(data)) {
                        $t.showSite(data);
                        if (data.IsWAAInstalled) {
                            $t.loadApp(data.Url);
                        }
                    }
                },
                error: function (_XMLHttpRequest, _textStatus, _errorThrown) {
                    $t.buildRefreshBtn(this);
                    $t.alert("Refresh site failed, please click Refresh icon for site "+this+" after a while.");
                },
                complete: function (_XMLHttpRequest, _textStatus) {
                    $t.log("Load site completed. " + _textStatus);                    
                }

            })
        };

        // Load the Site from Cache
        eat.loadSites = function () {
            $t.mask.open("System is loading site collections, please stand by.");

            var startIndex = $("#link_sc_more").attr("Index");
            if (typeof startIndex == "undefined" || startIndex == null) {
                startIndex = 0;
            }

            var search = $("#link_sc_more").attr("Search");
            var _url = search == "1" ? "/Default/SearchByIndex" : "/Default/LoadSites";
            //Before loading sites, make 'more' button disabled
            $("#link_sc_more").addClass("btn_gray").attr('disabled', true);
            $t.log("Load sites start from index: " + startIndex);
            $.ajax({
                type: "POST",
                cache: false,
                url: _url,
                data: { index: startIndex },
                success: function (data, _textStatus) {
                    data = $t.parseData(data);
                    if ($t.checkData(data)) {
                        if (data.Total > startIndex) {
                            $t.logger.info("Load sites", "Loaded sites from backend, " + _textStatus + ". Count: " + (data.Data == null ? 0 : data.Data.length));
                            //Show sites on the table
                            $t.showSites(data);
                            $("#link_sc_more").removeClass("btn_gray").removeAttr('disabled');
                            $("#link_sc_more").addClass("btn_orange");
                        } else {
                            $("#link_sc_more").addClass("btn_gray").attr('disabled', true);
                        }

                    } else {
                        $t.alert("Load sites failed. " + (data && data.MsgContent) ? data.MsgContent : "");
                    }
                },
                error: function (_XMLHttpRequest, _textStatus, _errorThrown) {                    
                    if ($(_XMLHttpRequest.responseText) && $(_XMLHttpRequest.responseText).length > 1 && $(_XMLHttpRequest.responseText)[1].tagName=="TITLE")
                    {
                        $t.alert($(_XMLHttpRequest.responseText)[1].innerText);
                    }
                    else{
                        $t.alert("Load sites failed.");
                    }
                },
                complete: function (_XMLHttpRequest, _textStatus) {
                    $t.mask.close();
                    $t.log("Load sites completed. " + _textStatus);
                }

            })

        };

        // Search the Site from Cache
        eat.search = function () {
            //Before search, update the title
            $t.updateSearchTitle();
            var _conditionUrl = $("#searchText").val();
            //validate special characters in the query condition
            var reg = new RegExp("[<>]");
            if (reg.test(_conditionUrl)) {
                $t.alert("Search query is limited to alpha-numeric and supported special characters. The following special characters are not supported: <>");
                return;
            }
            // set condition waaisInstalled
            var _conditionInstalled = "";
            if ($("#ckb_waa_installed").is(":checked")) {
                if (!$("#ckb_waa_not_installed").is(":checked"))
                {
                    _conditionInstalled = "1";
                }
            }
            else {
                if ($("#ckb_waa_not_installed").is(":checked"))
                {
                    _conditionInstalled = "0";
                }
            }            
            // set condition waa attribute list
            var conditionArr = $t.getConditions();

            //Initialize the site collection table
            $("#sc_table").find("tbody").empty();
            $("#link_sc_more").removeAttr("Index");
            $("#link_sc_more").removeAttr("Authed");
            $("#link_sc_more").attr("Search", "1");
            $("#ckb_sc_all").prop("checked", false);

            //Check the condition, if it is empty then load default values            
            if ($t.isEmpty(_conditionUrl) && $t.isEmpty(_conditionInstalled) && conditionArr.length==0) {
                //attributes should be undefined when the condition is null
                $("#link_sc_more").removeAttr("Index");
                $("#link_sc_more").removeAttr("Authed");
                $("#link_sc_more").removeAttr("Search");
                $t.loadSites();
                return;
            }            

            $t.mask.open("System is loading site collections, please stand by.");
                        
            $.ajax({
                type: "POST",
                cache: false,
                url: "/Default/SearchByCondition",
                data: { searchText: _conditionUrl, waaIsInstalled: _conditionInstalled, attributeList: conditionArr },
                success: function (data, _textStatus) {                    
                    data = $t.parseData(data);
                    if ($t.checkData(data)) {
                        $t.logger.info("Load sites", "Loaded sites from backend, " + _textStatus + ". Count: " + (data.Data == null ? 0 : data.Data.length));
                        //Show sites on the table
                        $t.showSites(data);
                    } else {
                        $t.alert("Load sites failed. " + (data && data.MsgContent) ? data.MsgContent : "");
                    }
                },
                error: function (_XMLHttpRequest, _textStatus, _errorThrown) {                    
                    $t.alert("Load sites failed.");
                },
                complete: function (_XMLHttpRequest, _textStatus) {
                    $t.mask.close();
                    $t.batch.check();
                    $t.log("Load sites completed. " + _textStatus);
                }

            })

        };                

        //Cache the site from service
        eat.cacheSite = function (_site) {
            _site.params = {
                previousEnabledState: false,
                createCustomAction: false,
                configs: {},
                sharePointCfg: {},
                cfgIds: {},
                webtrendsLoadJs: []
            };

            $t.globals.siteCollections[_site.Url] = _site;
            return _site;
        };

        //Show Sites
        eat.showSites = function (_pagination) {
            $t.log("Loaded the site collections, show them on page.")
            $t.showUnauthorisedSites(_pagination);
            
            var _sites = _pagination.Data;
            if (_sites != null && _sites.length > 0) {
                $("#link_sc_more").removeClass("btn_gray").removeAttr('disabled');
                //Only if the data is not null, update the Index value
                var count = $("#link_sc_more").attr("Index");
                if (typeof count == "undefined" || count == null) {
                    $("#link_sc_more").attr("Index", _pagination.Count);
                }
                else {
                    $("#link_sc_more").attr("Index", _pagination.Count + parseInt(count));
                }

                //record authed count
                var authedCount = $("#link_sc_more").attr("Authed");
                if (typeof authedCount == "undefined" || authedCount == null) {
                    $("#link_sc_more").attr("Authed", _pagination.Authed);
                }
                else {
                    $("#link_sc_more").attr("Authed", _pagination.Authed + parseInt(authedCount));
                }

                for (var i in _sites) {
                    //Cache the site collections                
                    var _site = $t.cacheSite(_sites[i]);

                    var siteRow = $("<tr/>", {
                        id: 'WTSite_' + _site.Id + "_row",
                        waaIsInstalled: _site.IsWAAInstalled ? 1 : 0,
                        url: _site.Url,
                        class: 'site_row'
                    });

                    $t.buildSiteCols(_site, siteRow);

                    $("<td/>", {
                        class: 'sc_state',
                        id: 'WTSite_' + _site.Id + "_State",
                        text: ""
                    }).appendTo(siteRow);

                    $("#sc_table").append(siteRow);
                }

                var totalstr = ($("#link_sc_more").attr("Index") ? $("#link_sc_more").attr("Index") : "0");
                if (totalstr != "0") {
                    totalstr = "1-" + totalstr + " of " + _pagination.Total;
                }
                $("#sc_page_info").text("App Deployed:" + $("tr[waaIsInstalled=1]").length + " | Authorized:" + ($("#link_sc_more").attr("Authed") ? $("#link_sc_more").attr("Authed") : "0") + " | Total:" + totalstr);

                //Scroll to the bottom of the page if user click More button
                if (typeof count != "undefined" && count != null) {
                    window.scrollTo(0, document.body.scrollHeight);
                    //$(".sc_content").scrollTop($(".sc_content").prop('scrollHeight'));
                }

            } else {
                $("#link_sc_more").addClass("btn_gray").attr('disabled', true);
                var count = $("#link_sc_more").attr("Index");
                if (!count || parseInt(count) == 0) {
                    $("#sc_page_info").text("Total: 0 of 0");
                }
            }
            
            //After show all sites, start to watch the batch status
            $t.batch.launchWatch();
        };

        //Build the waiting icon in "WAA App" column
        eat.buildWaitingCol = function (__id) {
            $('#WTSite_' + __id + "_App").empty();
            $("<img/>", {
                class: "pg_err",
                alt: "Please wait.",
                src: "/images/loading2.gif"
            }).appendTo('#WTSite_' + __id + "_App");
        };

        //Build the columns for each site collection
        eat.buildSiteCols = function (_site, siteRow) {
            //Reset the row state
            siteRow.attr("waaIsInstalled", _site.IsWAAInstalled ? 1 : 0);

            $("<td/>", {
                click: function (event) {
                    $(this).find(":checkbox").click();                    
                }
            }).append($('<input />', {
                id: 'WTSite_' + _site.Id + "_enabled",
                class: 'ckb_sc',
                _url: _site.Url,
                waaIsInstalled: _site.IsWAAInstalled ? 1 : 0,
                type: 'checkbox',
                click: function (event) {
                    //Hide the popup dialog
                    $(".status_view_popup").hide();
                    event.stopPropagation();
                },
                change: function () {                    
                    if ($(this).is(":checked")) {
                        $(this).parents("tr").addClass("selected");
                    } else {
                        $(this).parents("tr").removeClass("selected");
                    }
                    $t.batch.check();
                }
            })).appendTo(siteRow);

            $("<td/>", {
                id: 'WTSite_' + _site.Id + "_Id",
                class: '',
                text: _site.Id
            }).appendTo(siteRow);

            $("<td/>", {
                id: 'WTSite_' + _site.Id + "_Url",
                class: '',
                text: _site.Url
            }).appendTo(siteRow);


            $("<td/>", {
                id: 'WTSite_' + _site.Id + "_Title",
                class: '',
                text: _site.Title
            }).appendTo(siteRow);

            $("<td/>", {
                id: 'WTSite_' + _site.Id + "_App",
                class: '',
                html: _site.IsWAAInstalled ? $("<a/>", {
                    class: "link_sc_config",
                    text: (_site.WAAVersion ? _site.WAAVersion : "Deployed"),//_site.WAAVersion todo
                    site: _site.Url
                }).click(function () {
                    var _url = $(this).attr("site");
                    $t.showConfigs(_url);
                    //var site = $t.globals.siteCollections[_url];
                    //$("#WTSite_" + site.Id + "_enabled").prop("checked", true);
                }) : $("<a/>", {
                    class: "link_sc_deploy",
                    text: "Click To Deploy",
                    site: _site.Url,
                    siteId: _site.Id
                }).click(function () {
                    var __url = $(this).attr("site");
                    var __id = $(this).attr("siteId");

                    $(".ckb_sc:checked").each(function () {
                        //$(this).prop("checked", false);
                        $(this).click();
                    });

                    if (!$('#WTSite_' + __id + "_enabled").is(":disabled")) {
                        //$('#WTSite_' + __id + "_enabled").prop("checked", true);
                        $('#WTSite_' + __id + "_enabled").click();
                    }
                    
                    $("#btn_apps").click();
                    //$t.deploy();
                    scrollTo(0, 0);
                })
            }).appendTo(siteRow);
           
            $("<td/>", {
                id: 'WTSite_' + _site.Id + "_Tracking",
                class: '',
                html: _site.IsWAAInstalled ? ( _site.IsWAAConfigLoaded ? $t.buildSwitch("tracking", _site) : 'Loading ...' ) : ''
            }).appendTo(siteRow);
           
            $("<td/>", {
                id: 'WTSite_' + _site.Id + "_Ups",
                class: '',
                html: _site.IsWAAInstalled ? ( _site.IsWAAConfigLoaded ? $t.buildSwitch("ups", _site) : 'Loading ...') : ''
            }).appendTo(siteRow);
        };

        eat.buildSwitch = function (stype, sobj) {
            if ($t.isNull(sobj)) return "";
            if (stype == "tracking") {
                return (sobj.TrackEnabled == '1' || sobj.TrackEnabled == true) ? $("<img/>", {
                    class: "switcher",
                    title: "Click To Disable Tracking",
                    objId: sobj.Url,
                    click: function () {
                        $t.globals.furtive = { stype: "tracking", cbId: "WTConfigenabled", cbVal: false, sobjId: $(this).attr("objId") };
                        $t.showConfigs($(this).attr("objId")); //actually it will not show
                    },
                    src: '../images/on.png'
                }) : $("<img/>", {
                    class: "switcher",
                    title: "Click To Enable Tracking",
                    objId: sobj.Url,
                    click: function () {
                        $t.globals.furtive = { stype: "tracking", cbId: "WTConfigenabled", cbVal: true, sobjId: $(this).attr("objId") };
                        $t.showConfigs($(this).attr("objId")); //actually it will not show
                    },
                    src: '../images/off.png'
                });
            } else if (stype == "ups") {
                return (sobj.UserProfileCollectionEnabled == '1' || sobj.UserProfileCollectionEnabled == true) ? $("<img/>", {
                    class: "switcher",
                    title: "Click To Disable User Profile Collection",
                    objId: sobj.Url,
                    click: function () {
                        $t.globals.furtive = { stype: "ups", cbId: "WTConfigextraUserInfo", cbVal: false, sobjId: $(this).attr("objId") };
                        $t.showConfigs($(this).attr("objId")); //actually it will not show
                    },
                    src: '../images/on.png'
                }) : $("<img/>", {
                    class: "switcher",
                    title: "Click To Enable User Profile Collection",
                    objId: sobj.Url,
                    click: function () {
                        $t.globals.furtive = { stype: "ups", cbId: "WTConfigextraUserInfo", cbVal: true, sobjId: $(this).attr("objId") };
                        $t.showConfigs($(this).attr("objId")); //actually it will not show
                    },
                    src: '../images/off.png'
                })
            }
        }

        //Refresh Site columns in the existing row
        eat.showSite = function (_site) {
            _site = $t.cacheSite(_site);

            var stateCol = $('#WTSite_' + _site.Id + "_State");
            var siteRow = $('#WTSite_' + _site.Id + "_row");
            //Empty the row and reset the class
            siteRow.empty();
            siteRow.attr("class", "site_row");

            $t.buildSiteCols(_site, siteRow);

            stateCol.appendTo(siteRow);
        };

        //Load version numbers in batch
        eat.loadVersionNumbers = function () {
            $t.cleanData();

            var urlArr = [];
            $(".ckb_sc:checked:not(:hidden)").each(function () {
                if ($(this).attr("waaIsInstalled") == "1") {
                    urlArr.push($(this).attr("_url"));
                }
            })

            if (urlArr.length >= 1) {
                $t.globals.batch.sites = urlArr;

                $t.log("Create Batch for loading version number. ");
                $t.mask.open("System is loading version number, please stand by.");
                //Uncheck 'Select All'
                $("#ckb_sc_all").prop("checked", false);

                $.ajax({
                    type: "POST",
                    async: false,
                    url: "/Default/CreateBatchLoadVersion",
                    contentType: "application/json",
                    data: JSON.stringify({ "urls": urlArr }),
                    success: function (data, _textStatus) {
                        data = $t.parseData(data);
                        if ($t.checkData(data)) {
                            //when batch is created, system will trigger batch action soon
                            if (data.BatchNo) {
                                $t.globals.batch.batchNo = data.BatchNo;
                                var _sites = $t.globals.batch.sites;
                                for (var i = 0; i < _sites.length; i++) {
                                    var _site = $t.globals.siteCollections[_sites[i]];
                                    //_site.InUse = true;
                                    //Reset
                                    $('#WTSite_' + _site.Id + "_enabled").prop("checked", false);
                                    //Use should not able to select this site to do anything again before it is installed.                                    
                                    $('#WTSite_' + _site.Id + "_enabled").attr("disabled", true);

                                    //Remove button
                                    $t.buildWaitingCol(_site.Id);
                                }

                                $t.batch.watchSites();
                            }
                        } else {
                            if (data.MsgType == "error" && data.MsgCatalog == 'failed') {
                                $t.alert(data.MsgContent);
                            }
                        }
                    },
                    error: function (_XMLHttpRequest, _textStatus, _errorThrown) {
                        $t.log("Error Create Batch for loading version number.");
                    },
                    complete: function (_XMLHttpRequest, _textStatus) {
                        $t.mask.close();
                        $t.log("Complete Batch for loading version number creation. " + _textStatus);
                    }

                })
            }

        };        

        //Load installed WAA version 
        eat.loadInstalledVersion = function (_site) {
            
            if (_site.IsWAAInstalled && (_site.WAAVersion == null || _site.WAAVersion == "")) {
                $t.log("Load WAA app's version ...");
                $t.mask.open("System is loading the installed app version, please stand by.");
                $.ajax({
                    cache: false,
                    url: "/Default/GetInstalledWAAVersion",
                    data: { url: _site.Url },
                    context: _site.Id ,
                    success: function (data) {
                        data = $t.parseData(data);
                        if ($t.checkData(data)) {
                            if (data.WAAVersion) {
                                $("#WTSite_" + this + "_App").find(".link_sc_config").text(data.WAAVersion);
                            }
                        }
                    },
                    complete: function (_XMLHttpRequest, _textStatus) {
                        $t.mask.close();
                        $t.log("Load WAA app version completed. " + _textStatus);
                    }
                });
            }
        };

        //Log Unauthorised Sites
        eat.showUnauthorisedSites = function (_pagination) {
            $t.log("Log Unauthorized Sites ...");
            var _sites = _pagination.OtherData;
            for (var i in _sites) {
                var _site = _sites[i];                
                $t.logger.warn("Unauthorized Site Collection", _site.Url);
            }

        };

        //Load the WAA app status, start the backend process
        eat.loadApp = function (_url) {
            $t.log("Load WAA app's status ...");
            $.ajax({
                cache: false,
                url: "/Default/LoadApp",
                data: { url: _url },
                success: function (data) {
                    $("#WTSite_" + data.Id + "_row").attr("TrackEnabled", data.TrackEnabled)
                        .attr("UserProfileCollectionEnabled", data.UserProfileCollectionEnabled)
                        .attr("ValidDCSEnabled", data.ValidDCSEnabled)
                        .attr("HostedJSEnabled", data.HostedJSEnabled);

                    //redraw the columns
                    if (data.IsWAAConfigLoaded == "1") {
                        $("#WTSite_" + data.Id + "_Tracking").html($t.buildSwitch("tracking", data));

                        $("#WTSite_" + data.Id + "_Ups").html($t.buildSwitch("ups", data));
                    }
                },
                complete: function (_XMLHttpRequest, _textStatus) {
                    //Do nothing  
                    $t.log("Load one WAA app status completed. " + _textStatus);
                }
            });
        };

        //Load Config information
        eat.getConfig = function (_url, _configType, _cb, _cp) {
            $t.log("Load Configuration from '" + _url + "', type is '" + _configType + "'");            
            $.ajax({
                cache: false,
                context: _url,
                //url: "/Default/" + ($t.globals.siteInUse == "" ? "GetDefaultConfig" : "GetConfig"),
                url: "/Default/GetConfig" ,
                data: { url: _url, configType: _configType },
                success: function (data) {
                    data = $t.parseData(data);
                    if ($t.checkData(data)) {
                        _cb(data, this);                        
                    } else {
                        if (data && data.MsgType == "error") {
                            $t.handleError(this, data.MsgContent);
                            $t.batch.trigger();//try next one
                        }
                        $t.mask.close();
                    }
                },
                error: function (_XMLHttpRequest, _textStatus, _errorThrown) {                    
                    $t.logger.err("Load Configuration Failed", "Configuration is loaded from " + this);
                    $t.alert("Load Configuration Failed.");
                    $t.pg.failed(this, "Load Configuration Failed.");
                    $t.mask.close();
                    $t.globals.furtive = false; // reset if error happens
                },
                complete: function (_XMLHttpRequest, _textStatus) {                    
                    $t.log("Load Configuration from '" + this + "' is completed. " + _textStatus);                    
                    if (_cp) _cp(this);
                }
            });
        };
    
        //Load Config information
        eat.getConfigForProcess = function (_url, _configType, _cb, _cp) {
            $t.log("Load Configuration for Process from '" + _url + "', type is '" + _configType + "'");
            $.ajax({
                cache: false,
                context: _url,
                url: "/Default/GetConfigForProcess",
                data: { url: _url, configType: _configType },
                success: function (data) {
                    data = $t.parseData(data);
                    if ($t.checkData(data)) {
                        _cb(data, this);
                    }
                },
                error: function (_XMLHttpRequest, _textStatus, _errorThrown) {
                    $t.logger.err("Load Configuration for Process Failed", "Configuration is loaded from " + this);
                    $t.alert("Load Configuration for Process Failed.");
                    $t.pg.failed(this, "Load Configuration for Process Failed.");
                    $t.mask.close();
                },
                complete: function (_XMLHttpRequest, _textStatus) {
                    $t.log("Load Configuration for Process from '" + this + "' is completed. " + _textStatus);                    
                    if (_cp) _cp(this);

                }
            });
        };

        // Update the configuration value
        eat.saveConfig = function (_url, _data, _configType, _cb, _cp) {
            $t.log("Update the configuration value for " + _url + " ConfigType:" + _configType);
            $.ajax({
                type: "POST",
                context: _url,
                url: "/Default/Save" + _configType,
                contentType: "application/json",
                data: JSON.stringify({ "url": _url, "data": _data, "batchNo": $t.globals.batch.batchNo }),
                success: function (data, _textStatus) {
                    data = $t.parseData(data);
                    if ($t.checkData(data)) {
                        if (data.MsgType == "info" && data.MsgCatalog == 'success') {
                            _cb(data, this);
                        }
                    } else {
                        if (data && data.MsgType == "error") {
                            $t.handleError(this, data.MsgContent);
                            $t.batch.trigger();//try next one
                        }
                        $t.mask.close();
                    }
                },
                error: function (_XMLHttpRequest, _textStatus, _errorThrown) {
                    $t.log("Error updating configuration value. XHR ResponseText:"+_XMLHttpRequest.responseText);
                    $t.handleError(this, _errorThrown);
                    $t.batch.trigger();//try next one
                    $t.mask.close();
                },
                complete: function (_XMLHttpRequest, _textStatus) {                    
                    $t.log("Update the configuration value completed. " + _textStatus);
                    if (_cp) _cp(this);
                }

            })
        };

        //Show all Config information
        eat.showConfigs = function (_url) {
            //show dialog
            $t.openConfig();
            $t.mask.open("System is loading the configurations, please stand by.");            
            $t.globals.siteInUse = _url
            $t.loadWebtrendsSPConfigValues(_url);

            //Update the download link for configuration export.
            $("#exportConfigLink").attr("href", "/Default/ExportConfigs?url=" + _url);
            //Show export button
            $("#exportConfig").show();
        };

        eat.getFirstSelected = function () {
            var _ckb = $(".ckb_sc:checked:not(:hidden):first");
            var _url = _ckb.attr("_url");
            var site = $t.globals.siteCollections[_url];
            if (site && site.IsWAAInstalled) {
                return _url;
            }
            return null;
        };

        // Added code to handle different versions 
        eat.wtSPVersionValue = function (value, _url) {
            //var version = _spPageContextInfo.siteClientTag.split("$$")[1].split(".")[0];
            var version = "16";
            var site = $t.globals.siteCollections[_url];
            if (site) {
                version = site.SPVersion.split(".")[0];
            }            
            
            try {
                var t = JSON.parse(value);
                if (typeof (t) == 'object') return t[version];
            } catch (e) {
                // Do nothing
            }
            return value;
        };

        /***** LOAD CONFIGURATION *****/

        // Load configuration values from wtconfig list and create form
        eat.loadWebtrendsSPConfigValues = function (_url) {
            $t.getConfig(_url, 'wtconfig', function (configEnumerator, _url) {
                var _params = $t.globals.siteCollections[_url].params;

                // Create array to hold dependencies
                var dependencies = {};
                var tabGroupNum = 0;
                for (var i in configEnumerator) {
                    // Select the current configuration value
                    var configItem = configEnumerator[i];
                    var configRow = false;

                    // Check the config data of one line, if current config is in the config data, it should be show in the same line with its related item
                    var configkey = configItem["WTConfigKey"];
                    var oneLineData = $t.getOneLineConfigData($t.wtConfigOnOneLine, configkey);
                    if (oneLineData) {
                        // Create the content of the cell
                        $t.createConfigRowContents(configItem, "#WTConfig" + oneLineData.BeginItemConfigKey + "_row", oneLineData.startWitheControl, _url);
                    }
                    else {
                        // Create a row for this item
                        var configRow = jQuery('<tr/>', {
                            id: 'WTConfig' + configItem["WTConfigKey"] + '_row',
                            class: 'config_setting_row'
                        });

                        // Create the content of the cell
                        $t.createConfigRowContents(configItem, configRow, false, _url);
                    }

                    // Determine dependencies
                    if (configItem["WTDependency"] != null) {
                        dependencies[configItem["WTConfigKey"]] = configItem["WTDependency"];
                    }
                    var tabGroup = configItem["WTConfigTabGroup"].replace(/ /g, "_");
                    // If the tab group isn't defined, define it
                    if ($("#" + tabGroup + "_tab").length == 0) {
                        // Create list item for tab
                        jQuery('<li />', {
                            id: tabGroup + "_tab",
                            html: jQuery('<a />', { href: "#" + tabGroup, text: configItem["WTConfigTabGroup"] })
                        }).appendTo("#tabs");
                    }

                    if ($("#" + tabGroup).length == 0) {                        
                        // Create div and table to contain content
                        var tabContent = jQuery('<div />', {
                            id: tabGroup,
                            html: jQuery('<table />', { id: tabGroup + "table" })
                        });
                        // Hide by default if not the first one
                        if (tabGroupNum > 0) tabContent.hide();
                        // Attach whole div to tab group area
                        tabContent.appendTo("#configTabs");
                        tabGroupNum++;
                    }
                    // Add row to the appropriate config group
                    if (configRow) {
                        configRow.appendTo("#" + tabGroup + "table");
                    }
                }
                // Wire up dependencies
                $.each(dependencies, function (index, value) {
                    // Hook up onChange event to each item with a dependency
                    $('#WTConfig' + value).click(function () {
                        // Hide row if check box is unchecked
                        $('#WTConfig' + index + '_row').toggle($('#WTConfig' + value).prop("checked"));
                    });
                    // Hide if the default is that the row is now selected
                    if (!$('#WTConfig' + value).prop("checked")) $('#WTConfig' + index + '_row').hide();
                });           

                //Call next method
                $t.loadWebtrendsSPUserProfileSettings(_url);
            })
            
        };

        // Get all data of the item that shows in the same line with its dependency item
        eat.getOneLineConfigData = function (oneLineConfig, configKey) {
            var oneLineConfigData = false;

            if (oneLineConfig && configKey) {
                var linesConfig = oneLineConfig.split(",");
                if (linesConfig && linesConfig.length > 0) {
                    $.each(linesConfig, function (key, value) {
                        if (value.indexOf(":" + configKey + ":") >= 0) {
                            var itemsData = value.split(":");
                            if (itemsData && itemsData.length > 2) {
                                oneLineConfigData = {};
                                oneLineConfigData.BeginItemConfigKey = itemsData[0];
                                oneLineConfigData.followItemConfigKey = itemsData[1];
                                oneLineConfigData.startWitheControl = itemsData[2] === "true";
                                return false;
                            }
                        }
                    });
                }
            }

            return oneLineConfigData;
        };

        // Create all contents of a new row
        eat.createConfigRowContents = function (configItem, configRow, startWithControl, _url) {
            if (startWithControl) {
                // First: append the element; Second: append the label
                $t.createCellElement(configItem, configRow, _url);
                $t.createCellLabel(configItem, configRow);
            }
            else {
                // First: append the label; Second: append the element
                $t.createCellLabel(configItem, configRow);
                $t.createCellElement(configItem, configRow, _url);
            }
        },

        // Create a label of the cell
        eat.createCellLabel = function (configItem, configRow) {
            // Create a table cell with a label for this item
            jQuery('<td />', {
                id: 'WTConfig' + configItem["WTConfigKey"] + '_label',
                class: 'config_setting_row_label',
                text: configItem["Title"]
            }).appendTo(configRow);
        };

        // Create a form element of the cell
        eat.createCellElement = function (configItem, configRow, _url) {
            var _params = $t.globals.siteCollections[_url].params;

            // Check the Config Field Type and create form element based on this
            var configFormElement = '';
            switch (configItem["WTConfigFieldType"]) {
                case 'Boolean':
                    configFormElement = $('<input />', {
                        id: 'WTConfig' + configItem["WTConfigKey"],
                        type: 'checkbox'
                    });
                    if ($t.wtSPVersionValue(configItem["WTConfigValue"], _url) == '1') configFormElement.prop("checked", true);
                    // Hide the Uninstall button if Webtrends is not enabled
                    if (configItem["WTConfigKey"] == 'enabled') {
                        if ($t.wtSPVersionValue(configItem["WTConfigValue"], _url) == '1') {
                            if ($t.globals.siteInUse) {
                                $("#uninstallwt").show();
                            } else {
                                $("#uninstallwt").hide();
                            }
                            _params.previousEnabledState = true;
                        } else {
                            $("#uninstallwt").hide();
                            _params.previousEnabledState = false;
                        }
                    }
                    break;
                case 'Enum':
                    configFormElement = jQuery('<select />', {
                        id: 'WTConfig' + configItem["WTConfigKey"]
                    });
                    // Add options to the select element
                    var options = configItem["WTValidValues"].split(',');
                    for (var i = 0; i < options.length; i++) {
                        var option = jQuery('<option />', {
                            value: options[i],
                            text: options[i]
                        });
                        // If the current option is the config value, mark it as select
                        if (options[i] == $t.wtSPVersionValue(configItem["WTConfigValue"], _url)) option.attr('selected', 'selected');
                        // Append to form element
                        option.appendTo(configFormElement);
                    }
                    break;
                default:
                    configFormElement = jQuery('<input />', {
                        id: 'WTConfig' + configItem["WTConfigKey"],
                        type: 'text',
                        value: $t.wtSPVersionValue(configItem["WTConfigValue"], _url)
                    });
                    break;
            }
            // Create a table cell for the item's form element
            jQuery('<td />', {
                html: configFormElement
            }).appendTo(configRow);
        };

        // Retrieve current values from User Profile list
        eat.loadWebtrendsSPUserProfileSettings = function (_url) {
            $t.getConfig(_url, 'WTUserProfile', function (userProfileEnumerator, _url) {
                $('#WTConfigextraUserInfo_label').attr('colspan', 2);

                if ($("#User_Profile" + "_tab").length == 0) {
                    // Create list item for tab
                    jQuery('<li />', {
                        id: "User_Profile" + "_tab",
                        html: jQuery('<a />', { href: "#User_Profile", text: "User Profile Collection" })
                    }).appendTo("#tabs");
                }

                if ($("#User_Profile").length == 0) {                    
                    // Create div and table to contain content
                    var tabContent = jQuery('<div />', {
                        id: "User_Profile",
                        html: jQuery('<table />', { id: "User_Profiletable" })
                    }).hide();
                    // Attach whole div to tab group area
                    tabContent.appendTo("#configTabs");
                }

                // Create row header
                var configRow = jQuery('<tr/>', {
                    id: "WTUPHeader",
                    class: 'config_setting_row_header'
                });
                // Create a table cell with a label for this item
                jQuery('<td />', {
                    class: 'config_setting_row',
                    text: 'Collect?'
                }).appendTo(configRow);
                jQuery('<td />', {
                    class: 'config_setting_row',
                    text: 'User Profile Property'
                }).appendTo(configRow);
                jQuery('<td />', {
                    class: 'config_setting_row',
                    text: 'Assigned Parameter'
                }).appendTo(configRow);
                jQuery('<td />', {
                    class: 'config_setting_row_example',
                    text: 'Example Value'
                }).appendTo(configRow);
                // Add row to the appropriate config group
                $('#WTConfigextraUserInfo').click(function () {
                    // Hide row if check box is unchecked
                    $('#WTUPHeader').toggle($('#WTConfigextraUserInfo').prop("checked"));
                });
                // Hide if the default is that the row is now selected
                configRow.appendTo("#User_Profiletable");
                // This would be difficult to set to it's own property. And we already collect it from the DOM
                var enabled = {};
                $.each(userProfileEnumerator, function (i, userProfile) {
                    var index = userProfile["Title"];
                    var value = userProfile["ExampleValue"];
                    if (value.indexOf('|') > -1) value = value.split('|')[value.split('|').length - 1];
                    // Create config row
                    var configRow = jQuery('<tr/>', {
                        id: 'WTUP' + index + '_row'
                    });
                    // Add check box to determine if this is to be collected
                    var configFormElement = jQuery('<input />', {
                        id: 'WTUPField' + index + '_enabled',
                        class: 'WTUPEnabled',
                        type: 'checkbox'
                    });
                    // Set enabled/disabled based on whether this is in the 
                    //enabled = (index in userProfileFields) ? userProfileFields[index].WTUserProfileEnabled : '0';
                    enabled = userProfile["WTUserProfileEnabled"];
                    if (enabled == '1') configFormElement.prop("checked", true);
                    // Create a table cell for the item's enabled setting
                    jQuery('<td />', {
                        html: configFormElement
                    }).appendTo(configRow);
                    // Create a table cell with a label for this item
                    jQuery('<td />', {
                        id: 'WTConfig' + index + '_label',
                        text: index
                    }).appendTo(configRow);
                    // Get the param value for this item
                    configFormElement = jQuery('<input />', {
                        id: 'WTUPField' + index + "_param",
                        type: 'text',
                        class: 'WTUPParam',
                        //value: (index in userProfileFields) ? userProfileFields[index].WTUserProfileParam : ''
                        value: userProfile["WTUserProfileParam"]
                    });
                    if (enabled != '1') configFormElement.prop("readOnly", true);
                    // Get the parameter setting
                    jQuery('<td />', {
                        html: configFormElement
                    }).appendTo(configRow);
                    // Get the sample value for this item
                    jQuery('<td />', {
                        class: 'config_setting_row_example',
                        text: value
                    }).appendTo(configRow);
                    // Make dependent on the extraUserInfo check box
                    $('#WTConfigextraUserInfo').click(function () {
                        // Hook up onChange event to each item with a dependency
                        $('#WTUP' + index + '_row').toggle($('#WTConfigextraUserInfo').prop("checked"));
                    });
                    // Hide if the default is that the row is now selected
                    if (!$('#WTConfigextraUserInfo').prop("checked")) $('#WTUP' + index + '_row').hide();
                    // Add row to the appropriate config group
                    configRow.appendTo("#User_Profiletable");
                });
                $t.prepopulateUserProfile();

                //Call next method                
                $t.loadWebtrendsSPPlugins(_url);
            });
            

        };

        // Load configuration information from ewtplugin list and create form
        eat.loadWebtrendsSPPlugins = function (_url) {
            $t.getConfig(_url, 'wtplugins', function (pluginEnumerator, _url) {
                // Define list of fields to query
                var fieldList = ['Title', 'WTPluginKey', 'WTPluginHosted', 'WTPluginEnabled', 'WTPluginFilename', 'WTPluginOptions'];

                if ($("#wtplugins" + "_tab").length == 0) {
                    // Create tab for plugins
                    jQuery('<li />', {
                        id: "wtplugins" + "_tab",
                        html: jQuery('<a />', {
                            href: "#wtplugins",
                            text: "Plugins"
                        })
                    }).appendTo("#tabs");
                }

                if ($("#wtplugins").length == 0) {
                    // Create div to hold plugin information, append to config tabs, and hide by default
                    var wtPlugins = jQuery('<div />', {
                        id: "wtplugins"
                    }).appendTo("#configTabs").hide();
                }
                // Cycle through the list items
                for (var key in pluginEnumerator) {
                    // Select the current configuration value
                    var plugin = pluginEnumerator[key];
                    // Create a fieldset for this plugin                         
                    var pluginInstance = jQuery('<fieldset/>', {
                        id: 'WTPlugin' + plugin["WTPluginKey"] + '_group',
                        class: 'plugin_group'
                    }).append(jQuery('<legend />', {
                        text: plugin["Title"],
                        id: 'WTPlugin' + plugin["WTPluginKey"] + '_legend',
                        click: function () {
                            if ($(this).parent().children().length == 2)
                                $(this).parent().find("table").toggle();
                        }
                    }));
                    
                    // Create a table to hold the configuration for this plugin
                    var pluginOpts = jQuery('<table />', { id: 'WTPlugin' + plugin["WTPluginKey"] + '_settings', class: 'WTPluginSettings' });
                    // Create an array of plugin fields to show in the UI
                    var pluginUIFields = ["WTPluginEnabled", "WTPluginFilename", "WTPluginOptions"];
                    for (var i = 0; i < fieldList.length; i++) {
                        // If this is one of the plugin fields we want to show in the UI
                        if (pluginUIFields.indexOf(fieldList[i]) > -1) {
                            // Create a table row
                            var pluginRow = jQuery('<tr />', { class: 'plugin_row' });
                            // Create a table cell for the row label
                            jQuery('<td />', {
                                text: fieldList[i].substring(8)
                            }).appendTo(pluginRow);
                            // Determine the content to place within the row
                            var rowContent = '';
                            switch (fieldList[i]) {
                                case 'WTPluginEnabled':
                                    rowContent = jQuery('<input />', {
                                        type: 'checkbox',
                                        id: 'WTPlugin' + plugin["WTPluginKey"] + "_enabled"
                                    });
                                    if (plugin["WTPluginEnabled"] == '1' || plugin["WTPluginEnabled"]) {
                                        $(rowContent).prop('checked', true);
                                        pluginOpts.addClass('enabled');
                                    }
                                    break;
                                case 'WTPluginFilename':
                                    var baseUrl = (plugin["WTPluginHosted"] == '1') ? '//s.webtrends.com/js/' : _url + "/WebtrendsAssets/";
                                    rowContent = jQuery('<span/>', {
                                        class: 'disabled_option',
                                        text: baseUrl + plugin["WTPluginFilename"]
                                    });
                                    break;
                                case 'WTPluginOptions':
                                    rowContent = jQuery('<textarea/>', {
                                        id: 'WTPlugin' + plugin["WTPluginKey"] + "_options",
                                        rows: 10,
                                        cols: 80
                                    });
                                    rowContent.val(plugin["WTPluginOptions"]);
                                    break;
                            }
                            // Append form element for plugin setting to plugin row
                            jQuery('<td />', {
                                html: rowContent
                            }).appendTo(pluginRow);
                            // Append plugin row to plugin table
                            pluginRow.appendTo(pluginOpts);
                        }

                    }
                    // Append table to field set
                    pluginOpts.appendTo(pluginInstance);
                    // Append fieldset to div
                    pluginInstance.appendTo(wtPlugins);
                }
                $("#configTabs").show().tabs().show().tabs("option", "active", 0);
                
            }, function () {
                $t.mask.close();
                //after configrations are loaded completely, set the configs and save them automatically if current state is furtive
                if ($t.globals.furtive) {
                    $("#" + $t.globals.furtive.cbId).prop("checked", $t.globals.furtive.cbVal);
                    $t.globals.furtive = false;

                    $t.saveConfigs();
                }
            });

        };

        //Attach click event to User Profile checkboxes
        eat.prepopulateUserProfile = function () {
            $("#User_Profiletable :checkbox[class='WTUPEnabled']").bind("click", function () {
                var _map = { "Department": "WT.shp_dept", "Title": "WT.shp_title", "Manager": "WT.shp_mgr", "Office": "WT.shp_office", "Location": "WT.shp_location" };
                var _id = $(this).attr("id");
                var _isChecked = $(this).is(":checked");
                var _idx = _id.substring("WTUPField".length, _id.indexOf("_enabled"));
                var _txt = $("#" + 'WTUPField' + _idx + "_param");
                if (_txt.length == 1) {
                    var _txt_val = _txt.val();
                    if (/^\s*$/.test(_txt_val)) {
                        if (_map[_idx]) {
                            _txt.val(_map[_idx]);
                        } else {
                            _txt.val("WT.ups_" + _idx.toLowerCase());
                        }
                    }

                    if (_isChecked) {
                        _txt.attr("readOnly", false);
                    } else {
                        _txt.attr("readOnly", true);
                    }
                }
            });
        };

        /***** SAVE DATA BACK TO LISTS *****/        

        // Save the Webtrends configurations        
        eat.saveConfigs = function () {
            $t.cleanData();

            var _url = $t.globals.siteInUse;
            var urlArr = [];
            if (_url == null || _url.length <= 0) {
                $(".ckb_sc:checked:not(:hidden)").each(function () {
                    if ($(this).attr("waaIsInstalled") == "1") {
                        urlArr.push($(this).attr("_url"));
                    } 
                })                
            } else {
                urlArr.push(_url);
            }

            if (urlArr.length > 1) {
                //Create Batch and set the batch number to globals
                $t.batch.create(urlArr);

                if ($t.globals.batch.batchNo == "") {
                    return;
                } else {
                    //If create successfully, then add other attributes.
                    $t.globals.batch.sites = urlArr;
                    $t.globals.batch.sampleURL = urlArr[0];
                    $t.globals.batch.sampleIdx = 0;                    
                }
            }
            
            $t.launchSaveConfigs(urlArr[0]);
          
            //for (var i = 0; i < urlArr.length; i++) {
            //    $t.pg.createByUrl(urlArr[i]);
            //    $t.pg.progressByUrl(urlArr[i], 1);//start progress
            //    $t.saveWebtrendsSPConfiguration(urlArr[i]);
            //}
                      
        };

        // Save the congurations and start progress bar
        eat.launchSaveConfigs = function (_url) {
            if (_url) {
                $t.pg.createByUrl(_url);
                $t.pg.progressByUrl(_url, 1);//start progress
                $t.saveWebtrendsSPConfiguration(_url);
            }            
        };        

        // Save the Webtrends configuration to the wtconfig list
        eat.saveWebtrendsSPConfiguration = function (_url) {
            $t.log("Save the Webtrends configuration to the wtconfig list");
            $t.mask.open("Saving configuration settings.");
            
            $t.getConfig(_url, 'wtconfig', function (_data, _url_) {
                var errMsgs = "";
                $.each(_data, function (n, value) {
                    // Get the current value for the config item
                    var currentConfigValue = '';
                    switch (value.WTConfigFieldType) {
                        case 'Boolean':
                            if ($('#WTConfig' + value.WTConfigKey).prop("checked")) {
                                currentConfigValue = '1';
                            } else {
                                currentConfigValue = '0';
                            }
                            break;
                        case 'Enum':
                            currentConfigValue = $('#WTConfig' + value.WTConfigKey).val();
                            break;
                        default:
                            currentConfigValue = $('#WTConfig' + value.WTConfigKey).val();
                            var validation = value.WTValidValues;
                            if (validation) {
                                try {
                                    var regex = new RegExp(validation);
                                    if (!regex.test(currentConfigValue)) {
                                        errMsgs += "The value of " + value.Title + " is invalid, please re-enter.";
                                    }
                                } catch (e) {
                                    errMsgs += "Error definition of validation for " + value.Title;
                                }
                            }
                            break;
                    }
                    value.WTConfigValue = currentConfigValue;
                });

                if (errMsgs != "") {
                    $t.handleError(_url_, errMsgs);
                    $t.mask.close();
                    $("#waaConfig").dialog("open");
                    $t.prompt.err(errMsgs);
                } else {
                    // Update the configuration value
                    $t.saveConfig(_url_, _data, "WTConfig", function (data, _url_) {
                        $t.pg.progressByUrl(_url, 2);
                        $t.saveWebtrendsSPUserProfileSettings(_url_);
                    })
                }
                
            });
        };

        // Save user profile configs
        eat.saveWebtrendsSPUserProfileSettings = function (_url) {
            $t.log("Saving Webtrends user profile configuration settings.");
            $t.mask.open("Saving Webtrends user profile configuration settings.");
            
            // Create object to hold form settings
            var profileSettings = {};
            $('input.WTUPEnabled').each(function (index, value) {
                // Get user profile field property name
                var fieldName = value.id.substring(9, value.id.length - 8);
                // Get user profile enabled setting
                profileSettings[fieldName] = {
                    enabled: $('#' + value.id).prop("checked") ? '1' : '0',
                    param: $('#WTUPField' + fieldName + '_param').val()
                };
            });

            $t.getConfig(_url, 'WTUserProfile', function (_data, _url_) {
                // Set user profile field values                 
                $.each(_data, function (index, value) {
                    var profileSetting = profileSettings[value.Title];
                    if (profileSetting) {
                        value.WTUserProfileEnabled = profileSetting.enabled;
                        value.WTUserProfileParam = profileSetting.param;
                    }                    
                });

                // Update the user profile value
                $t.saveConfig(_url_, _data, "WTUserProfile", function (data, _url_) {
                    $t.pg.progressByUrl(_url, 3);
                    $t.saveWebtrendsSPPlugins(_url_);
                })
            });            
        };

        // Save the configs for the Webtrends Plugins to the wtplugins list
        eat.saveWebtrendsSPPlugins = function (_url) {
            $t.log("Saving plugin settings.");
            $t.mask.open("Saving plugin settings.");

            $t.getConfig(_url, 'wtplugins', function (_data, _url_) {
                $.each(_data, function (index, value) {
                    // Get the plugin key
                    var pluginKey = value.WTPluginKey;
                    var pluginEnabled = ($('#WTPlugin' + pluginKey + "_enabled").prop('checked')) ? '1' : '0';
                    var pluginOptions = $('#WTPlugin' + pluginKey + "_options").val();
                    value.WTPluginEnabled = pluginEnabled;
                    value.WTPluginOptions = pluginOptions;
                });

                // Update the plugins value
                $t.saveConfig(_url_, _data, "WTPlugins", function (data, _url_) {
                    $t.pg.progressByUrl(_url, 4);
                    $t.processWebtrendsConfiguration(_url_);                                                        
                    
                }, function (_url_) {
                    //Reload the configuration
                    $t.loadApp(_url_);
                })
            });
        };

        // This will also build the script to inject into the pages.
        eat.processWebtrendsConfiguration = function (_url) {                       
            $t.mask.open("Processing updated configuration settings.");
            $t.getConfigForProcess(_url, 'wtconfig', function (configEnumerator, _url) {
                $t.pg.progressByUrl(_url, 5);
                var _params = $t.globals.siteCollections[_url].params;
                
                // Create array to hold dependencies
                var dependencies = {};
                var tabGroupNum = 0;
                for (var i in configEnumerator) {
                    // Select the current configuration value
                    var configItem = configEnumerator[i];
                    // Get the current value for the config item
                    var currentConfigValue = '';
                    switch (configItem["WTConfigFieldType"]) {
                        case 'Boolean':
                            currentConfigValue = ($t.wtSPVersionValue(configItem["WTConfigValue"], _url) == '1') ? 'true' : 'false';
                            break;
                        default:
                            currentConfigValue = "\"" + $t.wtSPVersionValue(configItem["WTConfigValue"], _url) + "\"";
                            if (configItem["WTConfigKey"] === 'desiredIpAddress' && configItem["WTValidValues"]) {
                                try {
                                    var regex = new RegExp(configItem["WTValidValues"]);
                                    if (!regex.test(["WTConfigValue"])) {
                                        currentConfigValue = "\"192.168.0.1\"";
                                    }
                                } catch (e) {
                                    currentConfigValue = "\"192.168.0.1\"";
                                }
                            }
                            break;
                    }
                    // Set the config value
                    var config = {
                        value: currentConfigValue,
                        dependency: configItem["WTDependency"],
                        include: configItem["WTIncludeInTag"] //(configItem["WTIncludeInTag"] == '1') ? true : false
                    };
                    // Add config value to the appropriate set of configs - User Profile and SharePoint are both in sp configs
                    if (configItem["WTConfigTabGroup"].toLowerCase() == 'sharepoint' || configItem["WTConfigTabGroup"].toLowerCase() == 'user profile') {
                        if (configItem["WTConfigKey"].substring(0, 4).toLowerCase() == 'ids_') {
                            _params.cfgIds[configItem["WTConfigKey"].substring(4)] = config;
                        } else {
                            _params.sharePointCfg[configItem["WTConfigKey"]] = config;
                        }
                    } else {
                        _params.configs[configItem["WTConfigKey"]] = config;
                    }

                }

                if (_params.configs.enabled.value.toLowerCase() == 'true') {
                    _params.createCustomAction = true;
                } else {
                    _params.createCustomAction = false;
                }

                var appInstanceScriptBase = _url + "/WebtrendsAssets/";
                // Create wt_sp_globals namespace and assign properties
                // webtrendsLoadJs holds the custom action's javascript code
                _params.webtrendsLoadJs = [];
                _params.webtrendsLoadJs.push("wt_sp_globals = window.wt_sp_globals ? window.wt_sp_globals : {};");

                _params.webtrendsLoadJs.push("wt_sp_globals.loadCount = 0;");
                // Set base URL
                // Change where to pull the webtrends.js based on the usehosted value
                try {
                    // Try/Catch in case someone removed the usehosted setting
                    if (_params.configs.usehosted.value == 'true') {
                        _params.webtrendsLoadJs.push("wt_sp_globals.baseTagUrl =\"//s.webtrends.com/js/\";");
                    } else {
                        _params.webtrendsLoadJs.push("wt_sp_globals.baseTagUrl =\"" + appInstanceScriptBase + "\";");
                    }
                } catch (e) {
                    _params.webtrendsLoadJs.push("wt_sp_globals.baseTagUrl =\"//s.webtrends.com/js/\";");
                }
                _params.webtrendsLoadJs.push("wt_sp_globals.url =\"" + appInstanceScriptBase + "\";");
                _params.webtrendsLoadJs.push("wt_sp_globals.configs = {");
                $.each(_params.configs, function (index, value) {
                    // Include in the tag configuration
                    if (value.include) {
                        // Make sure there are no dependencies or if there are dependencies, are they true?
                        if (value.dependency == null || value.dependency == '' || (value.dependency != null && value.dependency != '' && _params.configs[value.dependency].value == 'true')) {
                            _params.webtrendsLoadJs.push(index + " : " + value.value + ",");
                        }
                    }

                });
                _params.webtrendsLoadJs.push("homesite  : \"" + _url + "\",");                
                $t.processWebtrendsPlugins(_url);
            })
        };

        eat.processWebtrendsPlugins = function (_url) {
            $t.mask.open("Processing updated plugin settings.");
            $t.getConfigForProcess(_url, 'wtplugins', function (pluginEnumerator, _url) {
                $t.pg.progressByUrl(_url, 6);
                var _params = $t.globals.siteCollections[_url].params;

                _params.webtrendsLoadJs.push("plugins:{");
                // Adding this as it was reported an unnecessary comma caused problems
                var pluginsJSON = [];
                var pluginString = '';
                for (var key in pluginEnumerator) {
                    pluginString = '';
                    // Select the current plugin settings
                    if (key != 'ups' || (key == 'ups' && _params.sharePointCfg.extraUserInfo.value.toLowerCase() == 'true')) {
                        var pluginItem = pluginEnumerator[key];
                        // Get the plugin value from the list
                        pluginString += pluginItem["WTPluginKey"] + " : {";
                        var pluginPath = (pluginItem["WTPluginHosted"]) ? '"//s.webtrends.com/js/' + pluginItem["WTPluginFilename"] + '"' : 'wt_sp_globals.url + "' + pluginItem["WTPluginFilename"] + '"';
                        pluginString += "src:" + pluginPath;
                        if (pluginItem["WTPluginOptions"] != null && pluginItem["WTPluginOptions"] != '') {
                            pluginString += "," + pluginItem["WTPluginOptions"];
                        }
                        // Close this plugin's configs
                        pluginsJSON.push(pluginString + "}");
                    }
                }
                _params.webtrendsLoadJs.push(pluginsJSON.join(","));

                _params.webtrendsLoadJs.push("}};");
                // Add configs for SharePoint specifically
                // Yes, this is a little messy
                // Set Sharepoint configurations
                _params.webtrendsLoadJs.push("if (typeof(wt_sp_globals.configs.plugins.sp) != 'undefined') {");
                $.each(_params.sharePointCfg, function (index, value) {
                    // Include in the tag configuration
                    if (value.include) {
                        // Ignoring dependencies since more data won't cause a negative issue
                        _params.webtrendsLoadJs.push("wt_sp_globals.configs.plugins.sp." + index + " = " + value.value + ";");
                    }
                });
                // Collect Sharepoint IDs
                _params.webtrendsLoadJs.push("wt_sp_globals.configs.plugins.sp.ids = {};");
                $.each(_params.cfgIds, function (index, value) {
                    // Include in the tag configuration
                    if (value.include) {
                        // Ignoring dependencies since more data won't cause a negative issue
                        _params.webtrendsLoadJs.push("wt_sp_globals.configs.plugins.sp.ids." + index + " = " + value.value + ";");
                    }
                });
                _params.webtrendsLoadJs.push("};");
                // If we are collecting user information:
                if (_params.sharePointCfg.extraUserInfo.value.toLowerCase() == 'true') {
                    $t.processWebtrendsUserProfileSettings(_url);
                } else {
                    if (_params.createCustomAction) {//todo                        
                        $t.getSiteConfigValue(_url);
                    } else {
                        if (_params.previousEnabledState) {
                            $t.uninstallWebtrends(_url);
                        } else {
                            $t.pg.progressByUrl(_url, 8); //the last step
                            $t.mask.close();
                        }
                        //End to trigger batch
                        $t.batch.trigger();
                    }
                }

            });
        };

        eat.processWebtrendsUserProfileSettings = function (_url) {
            $t.mask.open("Processing updated user profile settings.");
            $t.getConfigForProcess(_url, 'WTUserProfile', function (userProfileEnumerator, _url) {
                $t.pg.progressByUrl(_url, 7);
                var _params = $t.globals.siteCollections[_url].params;
                
                _params.webtrendsLoadJs.push("wt_sp_globals.configs.plugins.ups = (wt_sp_globals.configs.plugins.ups) ? wt_sp_globals.configs.plugins.ups : {};");
                _params.webtrendsLoadJs.push("wt_sp_globals.configs.plugins.ups.userProfile = {");
                var userProfileConfigContainer = [];
                for (var key in userProfileEnumerator) {
                    // Select the current userProfile settings
                    var userProfileItem = userProfileEnumerator[key];
                    userProfileConfigContainer.push("'" + userProfileItem["Title"] + "':'" + userProfileItem["WTUserProfileParam"] + "'");
                }
                // Close script block and 
                _params.webtrendsLoadJs.push(userProfileConfigContainer.join(',') + "};");

                if (_params.createCustomAction) {                      
                    $t.getSiteConfigValue(_url);
                } else {
                    if (_params.previousEnabledState) {
                        $t.uninstallWebtrends(_url);
                    } else {
                        $t.pg.progressByUrl(_url, 8); //the last step
                        $t.mask.close();
                    }
                    //End to trigger batch
                    $t.batch.trigger();
                }
            })
        };

        eat.getSiteConfigValue = function (_url) {
            $t.log("Processing updated tracking settings for '" + _url + "'");
            $t.mask.open("Processing updated tracking settings.");
            $.ajax({
                cache: false,
                type: "POST",
                context: _url,
                url: "/Default/GetSiteConfigValue",
                data: { "url": _url, "webtrendsLoadJs": $t.globals.siteCollections[_url].params.webtrendsLoadJs.join("\n"), "batchNo": $t.globals.batch.batchNo },
                success: function (data) {
                    data = $t.parseData(data);
                    if ($t.checkData(data)) {
                        if (data.MsgType == "info" && data.MsgCatalog == 'success') {
                            $t.pg.progressByUrl(this, 8);
                            //TODO:
                            $("#uninstallwt").show();
                            $t.globals.siteCollections[this].params.previousEnabledState = true;
                        }
                    } else {                        
                        if (data && data.MsgType == "error") {
                            $t.handleError(this, data.MsgContent);
                        }                        
                    }
                    //End to trigger batch
                    $t.batch.trigger();
                },
                error: function (_XMLHttpRequest, _textStatus, _errorThrown) {
                    $t.logger.err("Processing updated tracking settings", "Get Site Config Value from " + this);
                    $t.handleError(this, "Error Processing updated tracking settings");
                },
                complete: function (_XMLHttpRequest, _textStatus) {
                    $t.log("Processing updated tracking settings for '" + this + "' is completed. " + _textStatus);
                    $t.mask.close();
                }
            });
        };

        /***** UNINSTALL APP *****/
        eat.uninstallWebtrends = function (_url) {
            
            if (typeof(_url) == 'undefined' || typeof(_url) == 'object') 
                _url = $t.globals.siteInUse;

            if (_url == null || _url.length <= 0) {
                $t.alert("No Site Colletion is selected.");
                return;
            }

            $t.log("Uninstalling Webtrends");
            $t.mask.open("Uninstalling Webtrends.");
            $t.pg.createByUrl(_url);
            //$t.pg.updateByUrl(_url, { max: 3 });
            $t.pg.progressByUrl(this, 0);
            $.ajax({
                cache: false,
                type: "POST",
                context: _url,
                url: "/Default/UninstallWebtrends",
                data: { "url": _url },
                success: function (data) {
                    data = $t.parseData(data);
                    if ($t.checkData(data)) {
                        if (data.MsgType == "info" && data.MsgCatalog == 'success') {                         
                            $("#WTConfigenabled").prop("checked", false);//todo
                            $t.pg.progressByUrl(this, 2);
                            $t.uninstallWebtrendsCustomAction(this);
                        }
                    } else {
                        if (data && data.MsgType == "error") {
                            $t.handleError(this, data.MsgContent);
                        }
                    }
                },
                error: function (_XMLHttpRequest, _textStatus, _errorThrown) {
                    $t.logger.err("Uninstalling Webtrends.", "Error updating configuration during uninstall from " + this);
                    $t.handleError(this, "Error updating configuration during uninstall");
                },
                complete: function (_XMLHttpRequest, _textStatus) {
                    $t.log("Uninstalling Webtrends for '" + this + "' is completed. " + _textStatus);                    
                }
            });
        };

        // Uninstall the assets that Webtrends installs
        eat.uninstallWebtrendsCustomAction = function (_url) {
            
            $t.log("Retracting Webtrends Custom Action.");
            $t.mask.open("Retracting Webtrends Custom Action.");
            $.ajax({
                cache: false,
                type: "POST",
                context: _url,
                url: "/Default/UninstallWebtrendsCustomAction",
                data: { "url": _url },
                success: function (data) {
                    data = $t.parseData(data);
                    if ($t.checkData(data)) {
                        if (data.MsgType == "info" && data.MsgCatalog == 'success') {                           
                            $t.pg.progressByUrl(this, 4);                            
                        }
                    } else {
                        if (data && data.MsgType == "error") {
                            $t.handleError(this, data.MsgContent);
                        }
                    }
                },
                error: function (_XMLHttpRequest, _textStatus, _errorThrown) {
                    $t.logger.err("Uninstalling Webtrends Custom Action.", "Error removing custom action during uninstall from " + this);
                    $t.handleError(this, "Error removing custom action during uninstall");
                },
                complete: function (_XMLHttpRequest, _textStatus) {
                    $t.log("Uninstalling Webtrends Custom Action for '" + this + "' is completed. " + _textStatus);
                    $t.uninstallWebtrendsAssetsList(this);
                }
            });
        };

        // Remove Webtrends Assets as part of the cleanup
        eat.uninstallWebtrendsAssetsList = function (_url) {
            $t.log("Retracting Webtrends Assets list on host web.");
            $t.mask.open("Retracting Webtrends Assets list on host web.");
            $.ajax({
                cache: false,
                type: "POST",
                context: _url,
                url: "/Default/UninstallWebtrendsAssetsList",
                data: { "url": _url },
                success: function (data) {
                    data = $t.parseData(data);
                    if ($t.checkData(data)) {
                        if (data.MsgType == "info" && data.MsgCatalog == 'success') {                            
                            $t.pg.progressByUrl(this, 8);//the last step
                        }
                    } else {
                        if (data && data.MsgType == "error") {
                            $t.handleError(this, data.MsgContent);
                        }
                    }
                },
                error: function (_XMLHttpRequest, _textStatus, _errorThrown) {
                    $t.logger.err("Uninstalling Webtrends Assets.", "Error removing Webtrends Assets from host web" + this);
                    $t.handleError(this, "Error removing Webtrends Assets");
                },
                complete: function (_XMLHttpRequest, _textStatus) {
                    $t.log("Uninstalling Webtrends Assets for '" + this + "' is completed. " + _textStatus);
                    
                    $t.globals.siteCollections[this].params.previousEnabledState = false;
                    $("#WTConfigenabled").prop("checked", false);
                    $('#uninstallwt').hide();
                    $t.refreshSite(this); // update the fields in list
                    $t.mask.close();
                }
            });
        };

        // Batch Deploy Webtrends Advanced Analytics
        eat.deploy = function (_appFileName) {
            $t.cleanData();

            var urlArr = [];
            $(".ckb_sc:checked:not(:hidden)").each(function () {
                if ($(this).attr("waaIsInstalled") == "0") {
                    urlArr.push($(this).attr("_url"));
                }
            })
            
            if (urlArr.length >= 1) {
                $t.globals.batch.sites = urlArr;

                $t.log("Create Batch for Deploy. ");
                $t.mask.open("System is creating batch deploy, please stand by.");
                //Uncheck 'Select All'
                $("#ckb_sc_all").prop("checked", false);

                $.ajax({
                    type: "POST",
                    async: false,
                    url: "/Default/CreateBatchDeploy",
                    contentType: "application/json",
                    data: JSON.stringify({ "urls": urlArr, "appName": _appFileName?_appFileName:"" }),
                    success: function (data, _textStatus) {
                        data = $t.parseData(data);
                        if ($t.checkData(data)) {
                            //Batch Deploy is different from Batch Config, 
                            //when batch deploy is created, system will trigger batch action soon
                            if (data.BatchNo) {
                                $t.globals.batch.batchNo = data.BatchNo;
                                var _sites = $t.globals.batch.sites;
                                for (var i = 0; i < _sites.length; i++) {
                                    var _site = $t.globals.siteCollections[_sites[i]];
                                    //_site.InUse = true;
                                    //Reset
                                    $('#WTSite_' + _site.Id + "_enabled").prop("checked", false);
                                    //Use should not able to select this site to do anything again before it is installed.                                    
                                    $('#WTSite_' + _site.Id + "_enabled").attr("disabled", true);

                                    //Don't know if waa is installed or not
                                    $('#WTSite_' + _site.Id + "_enabled").attr("waaIsInstalled", 2);
                                    $('#WTSite_' + _site.Id + "_row").attr("waaIsInstalled", 2);

                                    //Remove button
                                    $t.buildWaitingCol(_site.Id);
                                }
                                
                                $t.batch.watchSites();
                            }
                        } else {
                            if (data.MsgType == "error" && data.MsgCatalog == 'failed') {
                                $t.alert(data.MsgContent);
                            }
                        }
                    },
                    error: function (_XMLHttpRequest, _textStatus, _errorThrown) {
                        $t.log("Error Create Batch for Deploy.");
                    },
                    complete: function (_XMLHttpRequest, _textStatus) {
                        $t.mask.close();
                        $t.log("Complete Batch for Deploy Creation. " + _textStatus);
                    }

                })
            }

        };

        // Undeploy Webtrends Advanced Analytics
        eat.undeploy = function () {
            $t.cleanData();

            var urlArr = [];
            $(".ckb_sc:checked:not(:hidden)").each(function () {
                if ($(this).attr("waaIsInstalled") == "1") {
                    urlArr.push($(this).attr("_url"));
                }
            })

            if (urlArr.length >= 1) {
                $t.globals.batch.sites = urlArr;
                $t.log("Create Batch for Undeploy. ");
                $t.mask.open("System is creating batch undeploy, please stand by.");
                //Uncheck 'Select All'
                $("#ckb_sc_all").prop("checked", false);

                $.ajax({
                    type: "POST",
                    async: false,
                    url: "/Default/CreateBatchUndeploy",
                    contentType: "application/json",
                    data: JSON.stringify({ "urls": urlArr }),
                    success: function (data, _textStatus) {
                        data = $t.parseData(data);
                        if ($t.checkData(data)) {
                            //Batch Undeploy is different from Batch Config, 
                            //when batch Undeploy is created, system will trigger batch action soon
                            if (data.BatchNo) {
                                $t.globals.batch.batchNo = data.BatchNo;

                                var _sites = $t.globals.batch.sites;
                                for (var i = 0; i < _sites.length; i++) {                                    
                                    var _site = $t.globals.siteCollections[_sites[i]];
                                    //_site.InUse = true;
                                    //Reset
                                    $('#WTSite_' + _site.Id + "_enabled").prop("checked", false);
                                    //Use should not able to select this site to do anything again before it is installed.
                                    $('#WTSite_' + _site.Id + "_enabled").attr("disabled", true);

                                    //Don't know if waa is installed or not
                                    $('#WTSite_' + _site.Id + "_enabled").attr("waaIsInstalled", 2);
                                    $('#WTSite_' + _site.Id + "_row").attr("waaIsInstalled", 2);

                                    //Remove button
                                    $t.buildWaitingCol(_site.Id);
                                }

                                $t.batch.watchSites();
                            }
                        } else {
                            if (data.MsgType == "error" && data.MsgCatalog == 'failed') {
                                $t.alert(data.MsgContent);
                            }
                        }
                    },
                    error: function (_XMLHttpRequest, _textStatus, _errorThrown) {
                        $t.log("Error Create Batch for Undeploy.");
                    },
                    complete: function (_XMLHttpRequest, _textStatus) {
                        $t.mask.close();
                        $t.log("Complete Batch for Undeploy Creation. " + _textStatus);
                    }

                })
            }
        };

        /***** Batch Update for Configuration/Deploy *****/
        eat.batch = {
            //Attach events, this method can be invoked only once
            init: function () {
                //Batch Status
                $("#btn_batch_status").click(function () {
                    if ($("#batch_status_view_popup").is(":hidden")) {
                        $(".status_view_popup").hide();
                        $("#batch_status_view_popup").show();
                        $t.batch.load();

                    } else {
                        $("#batch_status_view_popup").hide();
                    }
                });

                $("#btn_batch_status_close").click(function () {
                    $("#batch_status_view_popup").hide();
                });
                
                //Refresh current batch
                $("#btn_batch_status_refresh").click($t.batch.fetch);

                //Retreive previous batch
                $("#btn_batch_status_prev").click($t.batch.fetch);

                //Retreive next batch
                $("#btn_batch_status_next").click($t.batch.fetch);

                //Batch config
                $("#btn_batch_config").click($t.batch.showConfig);

                //Batch Deploy
                $("#btn_batch_deploy").click($t.batch.showDeploy);
                
                //Batch Undeploy
                $("#btn_batch_undeploy").click($t.batch.showUndeploy);

                //Load version number
                $("#btn_installed_version").click($t.loadVersionNumbers);

                //Bulk upgrade
                $("#btn_upgrade").click($t.upgrade.showConfirm);
                
            },
            
            //Check for the both configuration and deploy
            //Return batch button status
            check: function () {                

                if ($("tr.site_row:not(:hidden)").length != 0 && $(".ckb_sc:checked:not(:hidden)").length != 0 && $("tr.site_row:not(:hidden)").length == ($(".ckb_sc:checked:not(:hidden)").length + $(".ckb_sc:disabled").length)) {
                    //all selected
                    $("#ckb_sc_all").prop("checked", true);
                } else {
                    $("#ckb_sc_all").prop("checked", false);
                }

                //Check if have the cheked site collection or not
                var defaultValid = $(".ckb_sc:checked:not(:hidden)").length > 0;
                var configValid = defaultValid;
                var deployValid = defaultValid;
                var _break = 0;
                $(".ckb_sc:checked:not(:hidden)").each(function () {
                    if ($(this).attr("waaIsInstalled") == "1") {
                        deployValid = false;
                    } else if ($(this).attr("waaIsInstalled") == "0") {
                        configValid = false;
                    } else if ($(this).attr("waaIsInstalled") == "2") {
                        deployValid = false;
                        configValid = false;
                    }
                    if (!deployValid && !configValid) return false;
                });

                if (defaultValid) {                    
                    $("#btn_batch_refresh").removeClass("btn_gray").removeAttr('disabled');
                } else {
                    $("#btn_batch_refresh").addClass("btn_gray").attr('disabled', true);
                }

                if (configValid) {
                    $("#btn_batch_config").removeClass("btn_gray").removeAttr('disabled');
                    $("#btn_batch_undeploy").removeClass("btn_gray").removeAttr('disabled');
                    $("#btn_installed_version").removeClass("btn_gray").removeAttr('disabled');
                    $("#btn_upgrade").removeClass("btn_gray").removeAttr('disabled');
                    $("#btn_batch_edit_version").removeClass("btn_gray").removeAttr('disabled');
                } else {
                    $("#btn_batch_config").addClass("btn_gray").attr('disabled', true);
                    $("#btn_batch_undeploy").addClass("btn_gray").attr('disabled', true);
                    $("#btn_installed_version").addClass("btn_gray").attr('disabled', true);
                    $("#btn_upgrade").addClass("btn_gray").attr('disabled', true);                    
                    $("#btn_batch_edit_version").addClass("btn_gray").attr('disabled', true);
                }

                if (deployValid) {
                    $("#btn_batch_deploy").removeClass("btn_gray").removeAttr('disabled');
                } else {
                    $("#btn_batch_deploy").addClass("btn_gray").attr('disabled', true);
                }
                return { config: configValid, deploy: deployValid };
            },

            //Show Batch Config information
            showConfig : function () {
                if ($t.batch.check().config) {
                    var _url = $t.getFirstSelected();
                    if (_url == null) {
                        $t.alert("The first site your selected is unavailable.");
                        return;
                    }

                    //show dialog
                    $t.openConfig();
                    $t.mask.open("System is loading the configurations, please stand by.");

                    $t.loadWebtrendsSPConfigValues(_url);
                }                        
            },

            //Show Batch Deploy information
            showDeploy : function () {
                //if (confirm("Click 'OK' to deploy the Webtrends Advanced Analytics to the selected Site Collection(s)?")) {
                //    $t.deploy();
                //    $t.batch.check();
                //}

                $("#btn_apps").click();
            },

            showUndeploy: function () {
                if (confirm("Click 'OK' to undeploy the Webtrends Advanced Analytics App from the selected Site Collection(s).")) {
                    if (confirm("This will delete all custom configurations for the Webtrends Advanced Analytics App for the selected Site Collection(s). Click 'OK' to proceed with the undeploy")) {
                        $t.undeploy();
                        $t.batch.check();
                    }                    
                }
            },

            // Load the latest batch
            load: function () {
                $t.log("Load the latest batch.");
                $("#batch_status_panel_content").empty().append($("<div class='progressbar'><div class='progress_label'>Loading ...</div></div>").progressbar({ value: false }));
                $.ajax({
                    type: "POST",
                    cache: false,
                    url: "/Default/GetLastBatch",
                    success: function (data, _textStatus) {
                        $("#batch_status_panel_content").empty();
                        data = $t.parseData(data);
                        if ($t.checkData(data)) {
                            //$.each(data, function (n, value) {
                                //$t.batch.inject(value);
                            //});
                            if (data.BatchNo) {
                                $t.batch.inject(data);
                            }
                        } else {
                            $("#batch_status_panel_content").append("No batch job found.");
                            $("#btn_batch_status_refresh, #btn_batch_status_prev, #btn_batch_status_next").addClass("btn_gray").attr("disabled", true).attr("batchNo", "");                            
                        }
                    },
                    error: function (_XMLHttpRequest, _textStatus, _errorThrown) {
                        //Do Nothing                    
                    },
                    complete: function (_XMLHttpRequest, _textStatus) {
                        $t.log("Load the latest batch completed. " + _textStatus);
                    }

                });
            },

            // Insert the batch into window
            inject: function (_batch) {
                var _msg = "";
                var _total = 0;
                var _success = 0;
                var _failed = 0;
                var _inprogress = 0;
                var _sites = $("<div class=\"tile_list scrollable\"></div>");
                $.each(_batch.DicBatchUpdateSite, function (i, value) {
                    if (value.Status == "Error" || value.Status == "Warning") {
                        var __msg = _msg = value.StatusDesc;
                        if (typeof _msg == 'object' && _msg instanceof Array) {
                            __msg = "";
                            for (var i = 0; i < _msg.length; i++) {
                                __msg += "[" + _msg[i].MsgSubject + " | " + _msg[i].MsgContent + "]";
                            }
                            //if it is object, then assign empty string to it.
                            _msg = "";
                        }
                        _sites.append($("<div/>", {
                            //class:(value.Status == "Warning"?"warn_title":"err_title"),
                            text: value.Url,
                            click: function () {
                                var err_div = $(this).children("div:last-child");
                                if (err_div.is(":hidden"))
                                    err_div.show();
                                else
                                    err_div.hide();
                            }
                        }).addClass(value.Status == "Warning" ? "warn_title" : "err_title").append($("<span></span>", {
                            text: (value.Status == null ? "" : value.Status)                            
                        })).append($("<div/>",{
                            //class: (value.Status == "Warning" ? "warn_info" : "err_info"),
                            text: __msg
                        }).addClass(value.Status == "Warning" ? "warn_info" : "err_info")));
                    } else {
                        _sites.append($("<div/>", {
                            text: value.Url
                        }).append($("<span></span>", {
                            text: (value.Status == null ? "" : value.Status)
                        })));
                    }
                    
                    //Here we can expect they are the same
                    if (value.Status == "Completed") {
                        _success++;
                    }
                    if (value.Status == "Error" || value.Status == "Warning") {
                        _failed++;
                    }
                    if (value.Status == "Inprogress") {
                        _inprogress++;
                    }
                    _total++;
                });
                _msg += "Total:" + _total + ", Completed:" + _success + " , Failed:" + _failed + ", In Progress:" + _inprogress;

                var tile = $("<div class=\"tile\"></div>");

                if (_batch.BatchState == "Completed" && _failed > 0) {
                    tile.append($("<input/>", {
                        type: "button",
                        value: "Retry",
                        batchNo: _batch.BatchNo,
                        click: function () {                            
                            $t.batch.retry($(this).attr("batchNo"));
                        }
                    }));
                }

                if (_batch.BatchState == "Completed") {
                    tile.append($("<input/>", {
                        type: "button",
                        value: "Delete",
                        batchNo: _batch.BatchNo,
                        click: function () {                            
                            $t.batch.remove($(this).attr("batchNo"));
                        }
                    }));
                }

                tile.append($("<h5>Batch No:</h5>").append($("<span></span>", {
                    text: _batch.BatchNo
                }))).append($("<h5>Batch Type:</h5>").append($("<span></span>", {
                    text: _batch.BatchType
                }))).append($("<p>Start Time:</p>").append($("<span></span>", {
                    text: _batch.BatchStartDate
                }))).append($("<p>End Time:</p>").append($("<span></span>", {
                    text: (_batch.BatchEndDate == null ? "" : _batch.BatchEndDate)
                }))).append($("<p>Status:</p>").append($("<span></span>", {
                    text: (_batch.BatchState == null ? "" : _batch.BatchState)
                }))).append($("<p>Detail:</p>").append($("<span></span>", {
                    text: _msg
                }))).append(_sites).prependTo($("#batch_status_panel_content"));

                if (_batch.PrevBatchNo != null && _batch.PrevBatchNo != "") {
                    $("#btn_batch_status_prev").removeClass("btn_gray").attr("disabled", false).attr("batchNo", _batch.PrevBatchNo);
                } else {
                    $("#btn_batch_status_prev").addClass("btn_gray").attr("disabled", true).attr("batchNo", _batch.PrevBatchNo);
                }

                if (_batch.NextBatchNo != null && _batch.NextBatchNo != "") {
                    $("#btn_batch_status_next").removeClass("btn_gray").attr("disabled", false).attr("batchNo", _batch.NextBatchNo);
                } else {
                    $("#btn_batch_status_next").addClass("btn_gray").attr("disabled", true).attr("batchNo", _batch.NextBatchNo);
                }

                $("#btn_batch_status_refresh").removeClass("btn_gray").attr("disabled", false).attr("batchNo", _batch.BatchNo);
                
            },

            //Run remove batch without any prompt
            performRemove: function (_batchNo) {
                $t.log("Delete Batch (" + _batchNo + ") start. ");
                $.ajax({
                    type: "POST",
                    cache: false,
                    url: "/Default/RemoveBatch",
                    data: { batchNo: _batchNo },
                    success: function (data, _textStatus) {
                        data = $t.parseData(data);
                        if ($t.checkData(data)) {
                            $t.batch.load();
                        } else {
                            $t.log("Delete Batch failed. ");
                        }
                    },
                    error: function (_XMLHttpRequest, _textStatus, _errorThrown) {
                        //$t.alert("Failed to Delete Batch.");
                        $t.log("Delete Batch failed. ");
                    },
                    complete: function (_XMLHttpRequest, _textStatus) {
                        $t.log("Delete Batch completed. " + _textStatus);
                    }

                });
            },

            // Remove the batch by its number
            remove: function (_batchNo) {
                if (confirm("Are you sure to delete this batch.")) {
                    $t.batch.performRemove(_batchNo);
                }
            },

            // Retrieve batch by no, the caller should have attribute "batchNo".
            fetch: function () {
                //Fetch it from button attribute
                var _batchNo = $(this).attr("batchNo");                    
                
                if (typeof _batchNo == "undefined" || _batchNo == null || _batchNo == "") {
                    return;
                }
                
                $t.batch.refresh(_batchNo);
            },

            // Refresh the batch status.
            refresh: function (_batchNo) {                
                if (typeof _batchNo == "undefined" || _batchNo == null || _batchNo == "") {
                    return;
                }
                

                $t.log("Load batch which number is " + _batchNo);
                $("#batch_status_panel_content").empty().append($("<div class='progressbar'><div class='progress_label'>Loading ...</div></div>").progressbar({ value: false }));


                $.ajax({
                    type: "POST",
                    cache: false,
                    url: "/Default/GetBatchByNo",
                    data: { batchNo: _batchNo },
                    success: function (data, _textStatus) {
                        $("#batch_status_panel_content").empty();
                        data = $t.parseData(data);
                        if ($t.checkData(data)) {
                            //$.each(data, function (n, value) {
                                //$t.batch.inject(value);
                            //});
                            if (data.BatchNo) {
                                $t.batch.inject(data);
                            }
                        } else {
                            $("#batch_status_panel_content").append("No batch job found.");
                            $("#btn_batch_status_refresh").addClass("btn_gray").attr("disabled", true).attr("batchNo", "");
                        }
                    },
                    error: function (_XMLHttpRequest, _textStatus, _errorThrown) {
                        //Do Nothing                    
                    },
                    complete: function (_XMLHttpRequest, _textStatus) {
                        $t.log("Load batch completed. " + _textStatus);
                    }

                });
            },

            // Create Batch Update the configuration value
            create : function (_urls) {
                if ($t.globals.batch.batchNo != "") {
                    $t.alert("There's a Batch Update not completed.");
                    return;
                }
                $t.log("Create Batch Update the configuration value. ");
                $.ajax({
                    type: "POST",
                    async: false,
                    url: "/Default/CreateBatch",
                    contentType: "application/json",
                    data: JSON.stringify({"urls":_urls}),
                    success: function (data, _textStatus) {
                        data = $t.parseData(data);
                        if ($t.checkData(data)) {
                            $t.globals.batch.batchNo = data.BatchNo;
                        } else {
                            if (data.MsgType == "error" && data.MsgCatalog == 'failed') {
                                $t.alert(data.MsgContent);
                            }
                        }
                    },
                    error: function (_XMLHttpRequest, _textStatus, _errorThrown) {
                        $t.log("Error Create Batch updating configuration value.");
                    },
                    complete: function (_XMLHttpRequest, _textStatus) {
                        $t.log("Create Batch Update the configuration value triggered. " + _textStatus);                    
                    }

                })
            },

            // Trigger Batch Configuration Update
            trigger : function () {
                //If the batch update is unnecessary, return.
                if ($t.globals.batch.batchNo == "") {
                    return;
                }
                //Check the site configuration is prepared.
                //If not, take the next site to save the configuration.
                if ($t.globals.batch.sampleState == "error") {
                    var idx = $t.globals.batch.sampleIdx + 1;
                    if ($t.globals.batch.sites.length >= (idx + 1)) {
                        $t.globals.batch.sampleURL = $t.globals.batch.sites[idx];
                        $t.globals.batch.sampleIdx = idx;
                        $t.globals.batch.sampleState = "";
                        $t.launchSaveConfigs($t.globals.batch.sites[idx]);
                    } else {
                        $t.batch.endBatch();
                        $t.alert("There's no site collection can be updated successfully. Please verify.");
                        $t.mask.close();
                    }
                    return;
                }                       

                $t.mask.close();
                $("#waaConfig").dialog("close");
                //TODO: Alert user to let him know the next step.
                $t.alert("Configuration validation and preparation have been done, batch job will run offline.");

                //Uncheck 'Select All'
                $("#ckb_sc_all").prop("checked", false);
                //Uncheck Selected row
                $(".ckb_sc").prop("checked", false);
                $(".ckb_sc").parents("tr").removeClass("selected");
                $t.batch.check();

                $t.log("Trigger Batch Update the configuration value. ");
                $.ajax({
                    type: "POST",
                    context: $t.globals.batch.batchNo,
                    url: "/Default/TriggerBatch",
                    contentType: "application/json",
                    data: JSON.stringify({ "batchNo": $t.globals.batch.batchNo }),
                    success: function (data, _textStatus) {                    
                        $t.batch.watchSites();
                    },
                    error: function (_XMLHttpRequest, _textStatus, _errorThrown) {
                        $t.log("Failed to trigger Batch updating configuration value.");
                    },
                    complete: function (_XMLHttpRequest, _textStatus) {
                        $t.log("Trigger Batch Update the configuration value triggered. " + _textStatus);
                        //Reset Batch Number
                        if (this == $t.globals.batch.batchNo)
                            $t.globals.batch.batchNo = "";
                    }

                });            
            },

            //Retry the batch execution, only failed items will run again.
            retry: function(_batchNo){            
                $t.log("Retry Batch Execution. ");
                $.ajax({
                    type: "POST",
                    context: _batchNo,
                    url: "/Default/TriggerBatch",
                    contentType: "application/json",
                    data: JSON.stringify({ "batchNo": _batchNo }),
                    success: function (data, _textStatus) {
                        $t.alert("The request of Retry has been sent out successfully.");
                        //After send request succesfully, refresh the window.
                        $t.batch.refresh(this);
                    },
                    error: function (_XMLHttpRequest, _textStatus, _errorThrown) {
                        $t.log("Failed to Retry Batch Execution.");
                    },
                    complete: function (_XMLHttpRequest, _textStatus) {
                        $t.log("Retry Batch Execution Completed. Status is " + _textStatus);                        
                    }

                });
            },

            // End batch,it should be called after this batch of each site configuration  have failed
            endBatch: function () {
                $.ajax({
                    cache: false,
                    type: "POST",
                    url: "/Default/EndBatch",
                    data: { "batchNo": $t.globals.batch.batchNo }
                });
            },
            
            //If batch site execution is completed(Error,Warning,Completed), this function should be called.
            //Only Deploy and Undeploy batch will need to build refresh button
            handleSiteStatus: function (_site) {
                $('#WTSite_' + _site.Id + "_enabled").attr("disabled", false);
                $('#WTSite_' + _site.Id + "_row").removeClass("selected");
                if(_site.BatchType == "Deploy" || _site.BatchType == "Undeploy" || _site.BatchType == "Upgrade") {
                    $t.buildRefreshBtn(_site.Url);
                    setTimeout("$t.buildWaitingCol('" + _site.Id + "');$t.loadSite('" + _site.Url + "')", 30000);
                } else if (_site.BatchType == "Configuration") {
                    $t.loadApp(_site.Url);
                } else if (_site.BatchType == "Version") { $t.loadSite(_site.Url); }
            },

            // Watch the Batch status
            watchSiteInBatch : function (_batchNo, _url) {
                if (_batchNo == "" || _url == "") {
                    return;
                }

                $t.log("Watch Site Batch Status, Call ... " + _url);
                $.ajax({
                    type: "POST",
                    context: { "batchNo": _batchNo, "url": _url },
                    url: "/Default/WatchSiteInBatch",
                    contentType: "application/json",
                    data: JSON.stringify({ "batchNo": _batchNo, "url": _url }),
                    success: function (data, _textStatus) {                    
                        data = $t.parseData(data);
                        if ($t.checkData(data)) {                       

                            $t.log("Watch Site Batch Status, Site '" + this.url + "' is in " + this.batchNo + ", Status:" + data.Status);
                            if (data.Status == "Error") {
                                $t.handleError(data.Url, data.StatusDesc);
                                $t.batch.handleSiteStatus(data);
                                return;
                            }

                            if (data.Status == "Warning") {
                                $t.handleWarning(data.Url, data.StatusDesc);
                                $t.batch.handleSiteStatus(data);
                                return;
                            }

                            //Sometimes the site has not been included into batch, delay to run again
                            if (data.MsgType == "info" && data.MsgCatalog == 'success') {
                                setTimeout("$t.batch.watchSiteInBatch('" + this.batchNo + "','" + this.url + "')", 2000);                                
                            }

                            $t.pg.createByUrl(data.Url);
                            $t.pg.progressByUrl(data.Url, data.CurrentStep);

                            if (data.Status == "Inprogress" || data.Status == null) {
                                //todo: disabled this site
                                setTimeout("$t.batch.watchSiteInBatch('" + this.batchNo + "','" + this.url + "')", 2000);
                            }

                            if (data.Status == "Completed") {                                
                                $t.batch.handleSiteStatus(data);
                            }
                        
                        } else {
                            if (data.MsgType == "error" && data.MsgCatalog == 'failed') {
                                $t.logger.appendResultMsg(data);
                            }
                        }                   

                    }
                })
            },

            watchSites: function () {
                var _sites = $t.globals.batch.sites;
                var _batchNo = $t.globals.batch.batchNo;
                //If the length is larger than 50, stop minitor
                if (_sites.length > 30) {
                    //$t.alert("To reduce the http connection, system will stop the monitor for each site collections. <br/> You can get result from <span style='font-weight:bold;'>Batch Status</span> window or <span style='font-weight:bold;'>Email</span>.");
                    $t.alert("You have started a large batch job. Real-time status updates have been disabled for this job. <br/>To see the current status of the batch job, use the <span style='font-weight:bold;'>Batch Status</span> window, or read batch job complete confirmation <span style='font-weight:bold;'>e-mail</span>");
                    for (var i in _sites) {
                        var _site = $t.globals.siteCollections[_sites[i]];
                        //$('#WTSite_' + _site.Id + "_App").empty();
                        $t.cleanBtns(_site.Id);
                        $('#WTSite_' + _site.Id + "_row").addClass("select-unable");
                    }
                    
                } else {
                    //Start monitor for each site
                    for (var i in _sites) {
                        $t.batch.watchSiteInBatch(_batchNo, _sites[i]);
                    }
                }
                
            },

            watchLoadConfigBatch: function () {
                $t.log("Watch LoadConfig Batch ... ");
                $.ajax({
                    type: "POST",
                    cache: false,
                    url: "/Default/GetLoadAppBatch",
                    success: function (data, _textStatus) {
                        $t.log("LoadConfig Batch object batchState: " + data.BatchState);
                        if (data != null && data.BatchState) {
                            
                            //Remove completed batch
                            if (data.BatchState == "Completed") {
                                $t.batch.performRemove(data.BatchNo);
                            }
                            
                            $.each(data.DicBatchUpdateSite, function (i, value) {
                                //update UI if site in batch is completed
                                if (value.Status == "Completed") {
                                    $t.log("LoadConfig Batch site URL: " + value.Url);
                                    var _site = $t.globals.siteCollections[value.Url];
                                    if (_site && _site.LoadAppBatchNo != data.BatchNo) {
                                        _site.LoadAppBatchNo = data.BatchNo;
                                        $t.loadSite(value.Url, false);
                                    }
                                } 
                            });

                            //Batch is not empty, so continue to watch
                            $t.batch.launchWatch();
                        }
                        
                    },
                    error: function (_XMLHttpRequest, _textStatus, _errorThrown) {
                        //nothing to do
                    },
                    complete: function (_XMLHttpRequest, _textStatus) {
                        $t.log("Watch LoadConfig Batch completed. " + _textStatus);
                    }
                })
            },

            launchWatch: function () {
                setTimeout(function () { $t.batch.watchLoadConfigBatch() }, 1000 * 10);
            }
        };

        /***** WAA app package manager *****/
        eat.versionManager = {
            //Bind Event, only can be invoked once
            init: function () {                
                $("#btn_apps").click(function () {
                    if ($("#apps_view_popup").is(":hidden")) {
                        $(".status_view_popup").hide();
                        $("#apps_view_popup").show();                        
                        $("#appPackages").val("");//remove the selected file path
                        $t.versionManager.load();

                    } else {
                        $("#apps_view_popup").hide();
                    }
                });
                $("#btn_upload_app").click($t.versionManager.open);
                $("#btn_apps_close").click(function () {
                    $("#apps_view_popup").hide();
                });

                //Init app file upload
                $("#versionUploadForm").ajaxForm({
                    beforeSubmit: function () {
                        if ($("#appPackages").get(0).files.length === 0) {
                            $t.alert("Please select the app file.");
                            return false;
                        }
                        return true;
                    },
                    beforeSend: function () {
                        $t.mask.open("System is uploading the file, please stand by.");
                    },
                    uploadProgress: function (event, position, total, percentComplete) {
                        //Do nothing
                    },
                    success: function (data) {
                        data = $t.parseData(data);
                        if ($t.checkData(data)) {
                            //If data is correct, close the upload dialog
                            $("#versionUploadDialog").dialog("close");
                            //Close the progress bar
                            $t.mask.close();
                            //Refresh dialog data
                            $t.versionManager.load();

                        } else {
                            //Close the progress bar if there's any issue
                            $t.mask.close();
                            //Show error message
                            $t.alert("Upload app file failed. " + (data && data.MsgContent) ? data.MsgContent : "");
                        }
                    },
                    complete: function (xhr) {
                        $t.log("Upload app file completed, " + xhr.responseText);
                    }
                });

                //Log the version number by manual
                $("#btn_batch_edit_version").click($t.versionManager.openVersionEditor);
                //dialog for edit versio number
                $("#versionNumEditor").dialog({
                    autoOpen: false,
                    resizable: true,
                    width: 360,
                    height: 90,
                    modal: true
                });
                //Save button
                $("#btn_save_vn").click($t.versionManager.saveVersionNums);
            },
            load: function () {                
                $t.log("Load WAA app package.");
                $("#app_panel_content").empty().prepend("<div class='appTitle'></div>");                
                //$t.versionManager.loadVersionNum(2, false);
                $t.versionManager.createPg();
                $t.versionManager.loadAppFiles();                
            },            
            createPg: function (_task_num) {
                if (!_task_num) {
                    _task_num = 1;
                }
                $("#app_panel_content .appTitle").empty();
                $("#app_panel_content").prepend($("<div class='progressbar' tasks='" + _task_num + "'><div class='progress_label'>Loading ...</div></div>").progressbar({ value: false }));
            },
            //_task_num is the number of the tasks will be run with this function
            //_reload is boolean value, if it is true, force to reload the value from SP
            loadVersionNum: function (_task_num, _reload) {
                if ($(":radio[name=selectRoute]:checked").val() == "Server") return;
                $t.versionManager.createPg(_task_num);
                $.ajax({
                    type: "POST",
                    cache: false,
                    url: "/Default/GetUploadedAppVersion",
                    data: { reload: _reload },
                    success: function (data, _textStatus) {
                        var pg = $("#app_panel_content .progressbar");
                        pg.attr("tasks", parseInt(pg.attr("tasks")) - 1);
                        if (pg.attr("tasks") == "0") {
                            pg.remove();
                        }
                        data = $t.parseData(data);
                        if ($t.checkData(data)) {
                            if (data.length == 0) {
                                $("#app_panel_content").append("No App Found In AppCatalog.");
                            } else {
                                $("#app_panel_content .appTitle").empty();
                                //first add AppCatalog uploaded WAA version
                                $("#app_panel_content .appTitle").append("The latest version of uploaded WAA in AppCatalog is ").
                                append("<span id='appCatalogWaaVersion' class='version_text'>" + data.version + "</span>").
                                append($("<img/>", {
                                    class: "versionRefresh",
                                    src: "/images/refresh-versionNum.png",
                                    click: function () {
                                        $t.versionManager.loadVersionNum(1, true);
                                    }
                                })).append($("<input/>", {
                                    type: "button",
                                    value: "Install It",
                                    title: "Install App version in AppCatalog",
                                    click: function () {
                                        if (confirm("Are you sure to upload the app in AppCatalog to the selected site(s)?")) {
                                            $t.deploy();
                                        }
                                    }
                                }));
                            }
                        } else {
                            $("#app_panel_content .appTitle").append(data.MsgContent);
                            $t.alert(data.MsgContent);
                        }
                    },
                    error: function (_XMLHttpRequest, _textStatus, _errorThrown) {
                        //Do Nothing                    
                    },
                    complete: function (_XMLHttpRequest, _textStatus) {
                        $("#app_panel_content .app_list div").removeClass("currentApp")
                        $("#div" + $("#appCatalogWaaVersion").text().replace(/[.]/g, "")).addClass("currentApp");
                        $t.mask.close();
                        $t.log("Load WAA APP Version from Appcatalog completed. " + _textStatus);                        
                    }
                });
            },
            loadAppFiles: function () {                
                $.ajax({
                    type: "POST",
                    cache: false,
                    url: "/Default/GetWAAPackages",
                    success: function (data, _textStatus) {
                        var pg = $("#app_panel_content .progressbar");
                        pg.attr("tasks", parseInt(pg.attr("tasks")) - 1);
                        if (pg.attr("tasks") == "0") {
                            pg.remove();
                        }
                        data = $t.parseData(data);
                        if ($t.checkData(data)) {
                            if (data.length == 0) {
                                $("#app_panel_content").append("No App Found.");
                            } else {
                                //list WAA app file 
                                var fileList = $("<div/>", {
                                    class: 'file_list app_list'
                                }).appendTo("#app_panel_content");
                                for (var i = 0; i < data.length; i++) {
                                    var index = data[i].split("\\").length;
                                    var fileName = data[i].split("\\")[index - 1];
                                    var displayFileName = fileName.split("-")[0] + ".app ";
                                    var fileNameArr = fileName.split("-"); var version = "";
                                    if (fileNameArr.length > 1)
                                        version = fileName.split("-")[1].replace(".app", "");
                                    var item = $("<div id='div" + version.replace(/[.]/g, "") + "'><img src=\"/images/waa.png\" /></div>").append("<span> " + displayFileName + "</span>").append(" <span class='versionNum'> " + version + "</span>")
                                        .append($("<input/>", {
                                        type: "button",
                                        value: "Delete",
                                        title: "Delete app file in the admin tool server",
                                        fileName: fileName,
                                        click: function () {
                                            $t.versionManager.remove($(this).attr("fileName"));
                                        }
                                    })).append($("<input/>", {
                                        type: "button",
                                        class: "btn_app_install",
                                        value: "Install It",
                                        title: "Install this app to the selected site directly.",
                                        fileName: fileName,
                                        click: function () {
                                            $t.versionManager.install($(this).attr("fileName"));
                                            $("#apps_view_popup").hide();
                                            $t.batch.check();
                                        }
                                    })).append($("<input/>", {
                                        type: "button",
                                        class: "btn_app_upgrade",
                                        value: "Upgrade To This Version",
                                        title: "Upgrade this app to the selected site directly.",
                                        fileName: fileName,
                                        click: function () {                                            
                                            $t.upgrade.create($(this).attr("fileName"));
                                            $("#apps_view_popup").hide();
                                            $t.batch.check();
                                        }
                                    })).hover(function () {
                                        //Once current row is onhover
                                        $(this).addClass("currentApp").siblings().removeClass("currentApp");
                                    });
                                    fileList.append(item);
                                }
                            }
                        }
                    },
                    error: function (_XMLHttpRequest, _textStatus, _errorThrown) {
                        //Do Nothing                    
                    },
                    complete: function (_XMLHttpRequest, _textStatus) {
                        $("#div" + $("#appCatalogWaaVersion").text().replace(/[.]/g, "")).addClass("currentApp");
                        $t.mask.close();
                        $t.log("Load WAA APP files completed. " + _textStatus);
                        //Check the button status
                        if ($("#btn_batch_deploy").is(":disabled")) {
                            $(".btn_app_install").hide();
                        } else {
                            $(".btn_app_install").show();
                        }

                        if ($("#btn_upgrade").is(":disabled")) {
                            $(".btn_app_upgrade").hide();
                        } else {
                            $(".btn_app_upgrade").show();
                        }
                    }
                });
            },
            remove: function (_fileName) {                
                if (confirm("Delete this app file?")) {
                    $t.log("Delete app file: " + _fileName);

                    $.ajax({
                        type: "POST",
                        cache: false,
                        url: "/Default/RemoveAppFile",
                        data: { deleteFileName: _fileName },
                        success: function (data, _textStatus) {
                            data = $t.parseData(data);
                            if ($t.checkData(data)) {
                                $t.versionManager.load();
                            } else {
                                $t.alert("Failed to delete app file.")
                            }
                        },
                        error: function (_XMLHttpRequest, _textStatus, _errorThrown) {
                            $t.alert("Failed to delete app file.");
                        },
                        complete: function (_XMLHttpRequest, _textStatus) {
                            $t.log("Delete app file completed. " + _textStatus);
                        }

                    })
                }
            },
            open: function () {
                $("#versionUploadDialog").dialog("open");
            },
            upload: function (_fileName) {
                if (confirm("Are you sure to upload this app file to AppCatalog site?")) {
                    $t.mask.open("System is uploading app file to AppCatalog site, please stand by.");
                    $t.log("upload app file to SharePoint: " + _fileName);
                    $.ajax({
                        type: "POST",
                        cache: false,
                        url: "/Default/UploadToAppCatalog",
                        data: { appFileName: _fileName },
                        success: function (data, _textStatus) {
                            data = $t.parseData(data);
                            if ($t.checkData(data)) {
                                //$t.versionManager.loadVersionNum(1,true);
                            } else {
                                if (data.MsgType == "error" && data.MsgCatalog == 'failed') {
                                    $t.alert(data.MsgContent);
                                }
                                else {
                                    $t.alert("Failed to upload app file to AppCatalog site.")
                                }
                            }
                        },
                        error: function (_XMLHttpRequest, _textStatus, _errorThrown) {
                            $t.alert("Failed to upload app file to AppCatalog site.");
                        },
                        complete: function (_XMLHttpRequest, _textStatus) {
                            $t.mask.close();
                            $t.log("Upload app file to AppCatalog site completed. " + _textStatus);
                        }
                    });
                }
            },
            install: function (_fileName) {
                if (confirm("Are you sure to install this app to the selected site(s)?")) {
                    //$t.mask.open("System is installing app to the selected site(s), please stand by.");
                    $t.log("install app to site(s): " + _fileName);
                    $t.deploy(_fileName);
                }
            },
            openVersionEditor: function () {
                $("#versionNumEditor").dialog("open");
            },
            saveVersionNums: function () {
                var urlArr = [];
                $(".ckb_sc:checked:not(:hidden)").each(function () {
                    if ($(this).attr("waaIsInstalled") == "1") {
                        urlArr.push($(this).attr("_url"));
                    }
                })

                if (urlArr.length >= 1) {

                    $t.log("Save version number to the selected site collections. ");
                    $t.mask.open("System is saving version number, please stand by.");                    

                    $.ajax({
                        type: "POST",
                        async: false,
                        url: "/Default/SaveVersionNumber",
                        contentType: "application/json",
                        data: JSON.stringify({ "urls": urlArr, "versionNumber": $("#versionNum").val() }),
                        success: function (data, _textStatus) {
                            data = $t.parseData(data);
                            if ($t.checkData(data)) {                                
                                $(".ckb_sc:checked:not(:hidden)").each(function () {
                                    if ($(this).attr("waaIsInstalled") == "1") {                                        
                                        $t.loadSite($(this).attr("_url"));
                                    }
                                })
                                
                            } else {
                                if (data.MsgType == "error" && data.MsgCatalog == 'failed') {
                                    $t.alert(data.MsgContent);
                                }
                            }
                        },
                        error: function (_XMLHttpRequest, _textStatus, _errorThrown) {
                            $t.log("Error to save version number to site collections.");
                        },
                        complete: function (_XMLHttpRequest, _textStatus) {
                            $t.mask.close();
                            $("#versionNumEditor").dialog("close");
                            $t.log("Complete saving version number to site collections. " + _textStatus);
                        }

                    })
                }
            }
        }

        /***** User Preference *****/
        eat.up = {
            //Bind Event, only can be invoked once
            init: function () {
                $("#btn_up").click(function () {
                    if ($("#up_view_popup").is(":hidden")) {
                        $(".status_view_popup").hide();
                        $("#up_view_popup").show();
                        $t.up.load();

                    } else {
                        $("#up_view_popup").hide();
                    }
                });
                $("#btn_up_save").click($t.up.save);
                $("#btn_up_close").click(function () {
                    $("#up_view_popup").hide();
                });
            },

            load: function () {
                $t.log("Load User Preference.");
                $("#up_panel_content").prepend($("<div class='progressbar'><div class='progress_label'>Loading ...</div></div>").progressbar({ value: false }));
                $.ajax({
                    type: "POST",
                    cache: false,
                    url: "/Default/GetUserPreference",
                    success: function (data, _textStatus) {                        
                        data = $t.parseData(data);
                        if ($t.checkData(data)) {
                            if (data.IsReceive) {
                                $("#up_receive").prop("checked", true);
                            } else {
                                $("#up_receive").prop("checked", false);
                            }
                            $("#up_email").val(data.MailAddress)
                            $("#up_setAppCatalogAddress").val(data.AppCatalogAddress);
                            $("#up_panel_content .progressbar").remove();
                        }
                    },
                    error: function (_XMLHttpRequest, _textStatus, _errorThrown) {
                        //Do Nothing                    
                    },
                    complete: function (_XMLHttpRequest, _textStatus) {
                        $t.log("Load User Preference completed. " + _textStatus);
                    }

                });
            },

            save: function () {
                var _isReceive=$("#up_receive").is(":checked") ? true : false;
                var _emailAddress = $("#up_email").val();
                var _appCatalogAddress = $("#up_setAppCatalogAddress").val();
                if (_emailAddress != '') {                    
                    var reg = /^([\w-]+(?:\.[\w-]+)*)@((?:[\w-]+\.)*\w[\w-]{0,66})\.([a-z]{2,6}(?:\.[a-z]{2})?)$/i;
                    if (!reg.test(_emailAddress)) {
                        $t.alert('Please input the correct mail address.');
                        return;
                    }
                } else {
                    if (_isReceive) {
                        $t.alert('Please input the mail address.');
                        return;
                    }
                }
                $t.mask.open("Save User Preference.");
                $t.log("Save User Preference.");
                $.ajax({
                    type: "POST",
                    data: { isReceive: _isReceive, emailAddress: _emailAddress, appCatalogAddress: _appCatalogAddress },
                    url: "/Default/SetUserPreference",
                    success: function (data, _textStatus) {
                        data = $t.parseData(data);
                        //check timeout error
                        $t.checkData(data);
                        //the result msg may be for success or error
                        if (data && data.MsgContent) {
                            $t.alert(data.MsgContent);
                        }                        
                    },
                    error: function (_XMLHttpRequest, _textStatus, _errorThrown) {
                        $t.logger.err("Save User Preference", "Failed to save User Preference");
                    },
                    complete: function (_XMLHttpRequest, _textStatus) {
                        $t.mask.close();
                        $t.log("Save User Preference completed. " + _textStatus);
                    }

                });
            }

        };

        /***** Batch Files Download *****/
        eat.download = {
            init: function () {
                $("#btn_download").click(function () {                    
                    if ($("#download_view_popup").is(":hidden")) {
                        $(".status_view_popup").hide();
                        $("#download_view_popup").show();
                        $t.download.load();
                    } else {
                        $("#download_view_popup").hide();
                    }
                });

                $("#btn_download_close").click(function () {
                    $("#download_view_popup").hide();
                });
            },

            load: function () {
                $t.log("Load batch files.");
               
                $.ajax({
                    type: "POST",
                    cache: false,
                    url: "/Default/GetBatchFiles",
                    success: function (data, _textStatus) {
                        var leftList = $("#batch_file_list").empty();
                        data = $t.parseData(data);
                        if ($t.checkData(data)) {
                            if (data.length == 0) {
                                leftList.append("No File Found.");
                            }
                            for (var i = 0; i < data.length; i++) {
                                var index = data[i].split("\\").length;
                                var fileName = data[i].split("\\")[index - 1];
                                //var openUrl = "window.open('/Default/DownloadBatchFile?fileName=" + fileName + "')";
                                if (fileName.indexOf(".txt") >= 0 && (fileName.indexOf("Archived") >= 0 || fileName.indexOf("Working") >= 0)) {
                                    var displayFileName = fileName.replace(".txt", "");
                                    if (displayFileName.indexOf("Archived") >= 0) {
                                        displayFileName = displayFileName.replace("Archived", "");
                                        var yy = displayFileName.substr(0, 4);
                                        var MM = displayFileName.substr(4, 2);
                                        var dd = displayFileName.substr(6, 2);
                                        var hh = displayFileName.substr(8, 2);
                                        var mm = displayFileName.substr(10, 2);
                                        var ss = displayFileName.substr(12, 2);
                                        displayFileName = MM + "/" + dd + "/" + yy + " " + hh + ":" + mm + ":" + ss;
                                    }

                                    var item = "<div><img src=\"/images/text.png\" /><a href='/Default/DownloadBatchFile?fileName=" + fileName + "'  >" + displayFileName + "</a></div>";
                                    if (displayFileName == "Working") {
                                        leftList.prepend(item);
                                    } else {
                                        leftList.append(item);
                                    }
                                    //$("#download_panel_content").append("<div><img src=\"/images/text.png\" /><a href='#' onclick=" + openUrl + " >" + displayFileName + "</a></div>");
                                    
                                }
                            }
                        }
                        leftList.prepend("<span>Batch Log List</span>");
                    },
                    error: function (_XMLHttpRequest, _textStatus, _errorThrown) {
                        //Do Nothing                    
                    },
                    complete: function (_XMLHttpRequest, _textStatus) {
                        $t.log("Load batch files completed. " + _textStatus);
                    }
                });
                $.ajax({
                    type: "POST",
                    cache: false,
                    url: "/Default/GetAuditLogFiles",
                    success: function (data, _textStatus) {
                        var rightList = $("#audit_file_list").empty();
                        data = $t.parseData(data);
                        if ($t.checkData(data)) {
                            if (data.length == 0) {
                                rightList.append("No File Found.");
                            }
                            for (var i = 0; i < data.length; i++) {
                                var index = data[i].split("\\").length;
                                var fileName = data[i].split("\\")[index - 1];                               
                                var item = "<div><img src=\"/images/text.png\" /><a href='/Default/DownloadAuditLogFile?fileName=" + fileName + "'  >" + fileName + "</a></div>";
                                rightList.append(item);
                            }
                        }
                        rightList.prepend("<span>Audit Log List</span>");
                    },
                    error: function (_XMLHttpRequest, _textStatus, _errorThrown) {
                        //Do Nothing                    
                    },
                    complete: function (_XMLHttpRequest, _textStatus) {
                        $t.log("Load batch files completed. " + _textStatus);
                    }
                });
            }
        };

        /***** Bulk Upgrade *****/
        eat.upgrade = {
            init: function () {
                //TODO: load version
            },

            showConfirm: function (_fileName) {
                //if (confirm("Click 'OK' to upgrade the Webtrends Advanced Analytics to the selected Site Collection(s)?")) {
                //    $t.upgrade.create(_fileName);
                //    $t.batch.check();
                //}
                $("#btn_apps").click();
            },

            create: function (_fileName) {
                $t.cleanData();

                var urlArr = [];
                $(".ckb_sc:checked:not(:hidden)").each(function () {
                    if ($(this).attr("waaIsInstalled") == "1") {
                        urlArr.push($(this).attr("_url"));
                    }
                })

                if (urlArr.length >= 1) {
                    $t.globals.batch.sites = urlArr;

                    $t.log("Create Batch for Bulk Upgrade. ");
                    $t.mask.open("System is creating bulk upgrade request, please stand by.");
                    //Uncheck 'Select All'
                    $("#ckb_sc_all").prop("checked", false);

                    $.ajax({
                        type: "POST",
                        async: false,
                        url: "/Default/CreateUpgradeRequest",
                        contentType: "application/json",
                        data: JSON.stringify({ "urls": urlArr, "appName": _fileName }),
                        success: function (data, _textStatus) {
                            data = $t.parseData(data);
                            if ($t.checkData(data)) {
                                //Bulk Upgrade is different from Batch Config, 
                                //when Bulk Upgrade is created, system will trigger batch action soon
                                if (data.BatchNo) {
                                    $t.globals.batch.batchNo = data.BatchNo;
                                    var _sites = $t.globals.batch.sites;
                                    for (var i = 0; i < _sites.length; i++) {
                                        var _site = $t.globals.siteCollections[_sites[i]];
                                        //_site.InUse = true;
                                        //Reset
                                        $('#WTSite_' + _site.Id + "_enabled").prop("checked", false);
                                        //Use should not able to select this site to do anything again before it is installed.                                    
                                        $('#WTSite_' + _site.Id + "_enabled").attr("disabled", true);

                                        //Don't know if waa is installed or not
                                        $('#WTSite_' + _site.Id + "_enabled").attr("waaIsInstalled", 2);
                                        $('#WTSite_' + _site.Id + "_row").attr("waaIsInstalled", 2);

                                        //Remove button
                                        $t.buildWaitingCol(_site.Id);
                                    }

                                    $t.batch.watchSites();
                                }
                            } else {
                                if (data.MsgType == "error" && data.MsgCatalog == 'failed') {
                                    $t.alert(data.MsgContent);
                                }
                            }
                        },
                        error: function (_XMLHttpRequest, _textStatus, _errorThrown) {
                            $t.log("Error Create Batch for Upgrade.");
                        },
                        complete: function (_XMLHttpRequest, _textStatus) {
                            $t.mask.close();
                            $t.log("Complete Batch for Upgrade Creation. " + _textStatus);
                        }

                    })
                }
            }
        };

        eat.synch = {
            init: function () {
                $("#btn_sync").click(function () {
                    $("#syncEditor").dialog("open");
                });
                //dialog for synch creation
                $("#syncEditor").dialog({
                    autoOpen: false,
                    resizable: true,
                    width: 340,
                    height: 90,
                    modal: true
                });
                //Save button
                $("#btn_save_sync").click($t.synch.createRequest);
            },

            createRequest: function () {
                $("#btn_save_sync").attr('disabled', true);
                $("#syncEditor").dialog("close");
                $t.mask.open("System is creating request for sync data from SharePoint, please stand by.");
                var hours = $("#period_hours").val();
                $.ajax({
                    type: "POST",
                    url: "/Default/SyncData",                    
                    contentType: "application/json",
                    context: { "hours": hours },
                    data: JSON.stringify({ "intervalHour": hours }),
                    success: function (data, _textStatus) {
                        data = $t.parseData(data);
                        if ($t.checkData(data)) {
                            if (this.hours == 0) {
                                $t.alert("Sync site collections from SharePoint successfully. ");
                                $t.search();
                            } else {
                                $t.alert("Sync request is created successfully, system will send out Email notification once it is completed. ");
                            }
                            
                        } else {
                            if (data.MsgType == "error" && data.MsgCatalog == 'failed') {
                                $t.alert(data.MsgContent);
                            }
                        }
                        
                    },
                    error: function (_XMLHttpRequest, _textStatus, _errorThrown) {
                        $t.log("Error Create sync data.");
                    },
                    complete: function (_XMLHttpRequest, _textStatus) {
                        $("#btn_save_sync").removeAttr('disabled');
                        $t.mask.close();
                        $t.log("Complete sync site collections from SharePoint. " + _textStatus);
                    }

                })
            }
        }

        return eat;
    })();

    _win.$t = $t;
})(window);

//Update the send method in XMLHttpRequest to add the nonce in header
var send = XMLHttpRequest.prototype.send;
XMLHttpRequest.prototype.send = function (data) {
    this.setRequestHeader('X-Requested-With', 'XMLHttpRequest');
    this.setRequestHeader('nonce', getCookie("nonce"));
    return send.apply(this, arguments);
}

//Get cookie from document
function getCookie(cname) {
    var name = cname + "=";
    var ca = document.cookie.split(';');
    for (var i = 0; i < ca.length; i++) {
        var c = ca[i];
        while (c.charAt(0) == ' ') c = c.substring(1);
        if (c.indexOf(name) == 0) return c.substring(name.length, c.length);
    }
    return "";
}


$(function () { $t.init(); })
